@@ -112,20 +112,27 @@ from simba.ui.pop_ups.set_machine_model_parameters_pop_up import \
 from simba.ui.pop_ups.severity_analysis_pop_up import AnalyzeSeverityPopUp
 from simba.ui.pop_ups.smoothing_interpolation_pop_up import (InterpolatePopUp,
                                                              SmoothingPopUp)
+from simba.ui.pop_ups.spontaneous_alternation_pop_up import \
+    SpontaneousAlternationPopUp
 from simba.ui.pop_ups.subset_feature_extractor_pop_up import \
     FeatureSubsetExtractorPopUp
 from simba.ui.pop_ups.third_party_annotator_appender_pop_up import \
     ThirdPartyAnnotatorAppenderPopUp
 from simba.ui.pop_ups.validation_plot_pop_up import ValidationVideoPopUp
 from simba.ui.pop_ups.video_processing_pop_up import (
-    CalculatePixelsPerMMInVideoPopUp, ChangeFpsMultipleVideosPopUp,
-    ChangeFpsSingleVideoPopUp, ChangeImageFormatPopUp, CLAHEPopUp,
-    ClipVideoPopUp, ConcatenatingVideosPopUp, ConcatenatorPopUp,
-    ConvertROIDefinitionsPopUp, ConvertVideoPopUp, CreateGIFPopUP,
-    CropVideoCirclesPopUp, CropVideoPolygonsPopUp, CropVideoPopUp,
+    BrightnessContrastPopUp, CalculatePixelsPerMMInVideoPopUp,
+    ChangeFpsMultipleVideosPopUp, ChangeFpsSingleVideoPopUp, CLAHEPopUp,
+    ClipSingleVideoByFrameNumbers, ClipVideoPopUp, ConcatenatingVideosPopUp,
+    ConcatenatorPopUp, Convert2AVIPopUp, Convert2bmpPopUp, Convert2jpegPopUp,
+    Convert2MOVPopUp, Convert2MP4PopUp, Convert2PNGPopUp, Convert2TIFFPopUp,
+    Convert2WEBMPopUp, Convert2WEBPPopUp, ConvertROIDefinitionsPopUp,
+    CreateGIFPopUP, CropVideoCirclesPopUp, CropVideoPolygonsPopUp,
+    CropVideoPopUp, DownsampleMultipleVideosPopUp, DownsampleSingleVideoPopUp,
     DownsampleVideoPopUp, ExtractAllFramesPopUp, ExtractAnnotationFramesPopUp,
     ExtractSEQFramesPopUp, ExtractSpecificFramesPopUp,
     GreyscaleSingleVideoPopUp, ImportFrameDirectoryPopUp,
+    InitiateClipMultipleVideosByFrameNumbersPopUp,
+    InitiateClipMultipleVideosByTimestampsPopUp, InteractiveClahePopUp,
     MergeFrames2VideoPopUp, MultiCropPopUp, MultiShortenPopUp,
     SuperImposeFrameCountPopUp, VideoRotatorPopUp, VideoTemporalJoinPopUp)
 from simba.ui.pop_ups.visualize_pose_in_dir_pop_up import \
@@ -135,7 +142,7 @@ from simba.ui.video_info_ui import VideoInfoTable
 from simba.utils.checks import (check_ffmpeg_available,
                                 check_file_exist_and_readable, check_int)
 from simba.utils.custom_feature_extractor import CustomFeatureExtractor
-from simba.utils.enums import OS, Defaults, Formats, TagNames
+from simba.utils.enums import OS, Defaults, Formats, Paths, TagNames
 from simba.utils.errors import InvalidInputError
 from simba.utils.lookups import (get_bp_config_code_class_pairs, get_emojis,
                                  get_icons_paths)
@@ -537,6 +544,14 @@ class SimbaProjectPopUp(ConfigReader, PopUpMixin):
             fg="grey",
             command=lambda: BooleanConditionalSlicerPopUp(config_path=self.config_path),
         )
+        spontaneous_alternation_pop_up_btn = Button(
+            processmovementdupLabel,
+            text="SPONTANEOUS ALTERNATION",
+            fg="navy",
+            command=lambda: threading.Thread(
+                target=SpontaneousAlternationPopUp(config_path=self.config_path)
+            ).start(),
+        )
 
         # organize
         processmovementdupLabel.grid(row=0, column=3, sticky=NW)
@@ -549,6 +564,7 @@ class SimbaProjectPopUp(ConfigReader, PopUpMixin):
         button_analyzeDirection_bp.grid(row=6, sticky=NW)
         button_visualizeDirection_bp.grid(row=7, sticky=NW)
         btn_agg_boolean_conditional_statistics.grid(row=8, sticky=NW)
+        spontaneous_alternation_pop_up_btn.grid(row=9, sticky=NW)
 
         label_outliercorrection = CreateLabelFrameWithIcon(
             parent=tab4,
@@ -895,9 +911,16 @@ class SimbaProjectPopUp(ConfigReader, PopUpMixin):
             color="blue",
             lblwidth=30,
             file_types=[("SimBA CSV", "*.csv"), ("SimBA PARQUET", "*.parquet")],
+            initialdir=os.path.join(
+                self.project_path, Paths.FEATURES_EXTRACTED_DIR.value
+            ),
         )
         self.modelfile = FileSelect(
-            label_model_validation, "SELECT MODEL FILE", color="blue", lblwidth=30
+            label_model_validation,
+            "SELECT MODEL FILE",
+            color="blue",
+            lblwidth=30,
+            initialdir=self.project_path,
         )
         button_runvalidmodel = Button(
             label_model_validation,
@@ -1444,8 +1467,9 @@ class SimbaProjectPopUp(ConfigReader, PopUpMixin):
         _ = MachineModelSettingsPopUp(config_path=self.config_path)
 
     def run_feature_extraction(self):
+        print("Running feature extraction...")
         print(
-            f"Pose-estimation body part setting for feature extraction: {str(self.animal_cnt)} animals {str(self.pose_setting)} body-parts"
+            f"Pose-estimation body part setting for feature extraction: {str(self.animal_cnt)} animals {str(self.pose_setting)} body-parts..."
         )
         feature_extractor_classes = get_bp_config_code_class_pairs()
         if self.user_defined_var.get():
@@ -1591,50 +1615,62 @@ class App(object):
         )
 
         menu.add_cascade(label="Tools", menu=video_process_menu)
-        video_process_menu.add_cascade(
-            label="Change fps...",
-            compound="left",
-            image=self.menu_icons["fps"]["img"],
-            menu=fps_menu,
-        )
-        format_menu = Menu(video_process_menu)
-        format_menu.add_command(
-            label="Change image file formats", command=ChangeImageFormatPopUp
-        )
-        format_menu.add_command(
-            label="Change video file formats", command=ConvertVideoPopUp
-        )
-        video_process_menu.add_cascade(
-            label="Change formats...",
-            compound="left",
-            image=self.menu_icons["convert"]["img"],
-            menu=format_menu,
-        )
-        video_process_menu.add_command(
-            label="CLAHE enhance video",
-            compound="left",
-            image=self.menu_icons["clahe"]["img"],
-            command=CLAHEPopUp,
+        video_process_menu.add_cascade(label="Change fps...", compound="left", image=self.menu_icons["fps"]["img"], menu=fps_menu)
+
+        clip_video_menu = Menu(menu)
+        clip_video_menu.add_command(label="Clip single video", command=ClipVideoPopUp)
+        clip_video_menu.add_command(
+            label="Clip multiple videos",
+            command=InitiateClipMultipleVideosByTimestampsPopUp,
         )
 
-        video_process_menu.add_command(
-            label="Clip videos",
-            compound="left",
-            image=self.menu_icons["clip"]["img"],
-            command=ClipVideoPopUp,
+        clip_video_menu.add_command(
+            label="Clip video into multiple videos", command=MultiShortenPopUp
         )
-        video_process_menu.add_command(
-            label="Clip video into multiple videos",
-            compound="left",
-            image=self.menu_icons["clip"]["img"],
-            command=MultiShortenPopUp,
+        clip_video_menu.add_command(
+            label="Clip single video by frame numbers",
+            command=ClipSingleVideoByFrameNumbers,
         )
+        clip_video_menu.add_command(
+            label="Clip multiple videos by frame numbers",
+            command=InitiateClipMultipleVideosByFrameNumbersPopUp,
+        )
+
         video_process_menu.add_cascade(
-            label="Concatenate multiple videos",
+            label="Clip videos...",
             compound="left",
-            image=self.menu_icons["concat"]["img"],
-            command=lambda: ConcatenatorPopUp(config_path=None),
+            image=self.menu_icons["clip"]["img"],
+            menu=clip_video_menu,
         )
+
+        crop_video_menu = Menu(menu)
+        crop_video_menu.add_command(label="Crop videos", compound="left", image=self.menu_icons["crop"]["img"], command=CropVideoPopUp)
+        crop_video_menu.add_command(label="Crop videos (circles)", compound="left", image=self.menu_icons["circle"]["img"], command=CropVideoCirclesPopUp)
+        crop_video_menu.add_command(label="Crop videos (polygons)", compound="left", image=self.menu_icons["polygon"]["img"], command=CropVideoPolygonsPopUp)
+        crop_video_menu.add_command(label="Multi-crop", compound="left", image=self.menu_icons["crop"]["img"], command=MultiCropPopUp)
+        video_process_menu.add_cascade(label="Crop videos...", compound="left", image=self.menu_icons["crop"]["img"], menu=crop_video_menu)
+
+        format_menu = Menu(video_process_menu)
+        img_format_menu = Menu(format_menu)
+        video_format_menu = Menu(format_menu)
+        img_format_menu.add_command(label="Convert image directory to PNG", command=Convert2PNGPopUp)
+        img_format_menu.add_command(label="Convert image directory to JPEG", command=Convert2jpegPopUp)
+        img_format_menu.add_command(label="Convert image directory to BMP", command=Convert2bmpPopUp)
+        img_format_menu.add_command(label="Convert image directory to TIFF", command=Convert2TIFFPopUp)
+        img_format_menu.add_command(label="Convert image directory to WEBP", command=Convert2WEBPPopUp)
+        video_format_menu.add_command(label="Convert videos to MP4", command=Convert2MP4PopUp)
+        video_format_menu.add_command(label="Convert videos to AVI", command=Convert2AVIPopUp)
+        video_format_menu.add_command(label="Convert videos to WEBM", command=Convert2WEBMPopUp)
+        video_format_menu.add_command(label="Convert videos to MOV", command=Convert2MOVPopUp)
+        format_menu.add_cascade(label="Change image formats...", compound="left", menu=img_format_menu)
+        format_menu.add_cascade(label="Change video formats...", compound="left", menu=video_format_menu)
+        video_process_menu.add_cascade(label="Change formats...", compound="left", image=self.menu_icons["convert"]["img"], menu=format_menu)
+        clahe_menu = Menu(video_process_menu)
+        clahe_menu.add_command(label="CLAHE enhance videos", command=CLAHEPopUp)
+        clahe_menu.add_command(label="Interactively CLAHE enhance videos", command=InteractiveClahePopUp)
+        video_process_menu.add_cascade(label="CLAHE enhance videos...", compound="left", image=self.menu_icons["clahe"]["img"], menu=clahe_menu)
+
+        video_process_menu.add_cascade(label="Concatenate multiple videos", compound="left", image=self.menu_icons["concat"]["img"], command=lambda: ConcatenatorPopUp(config_path=None))
         video_process_menu.add_cascade(
             label="Concatenate two videos",
             compound="left",
@@ -1654,12 +1690,9 @@ class App(object):
             command=lambda: ConvertROIDefinitionsPopUp(),
         )
         convert_data_menu = Menu(video_process_menu)
-        convert_data_menu.add_command(
-            label="Convert CSV to parquet", command=Csv2ParquetPopUp
-        )
-        convert_data_menu.add_command(
-            label="Convert parquet o CSV", command=Parquet2CsvPopUp
-        )
+        convert_data_menu.add_command(label="Convert CSV to parquet", command=Csv2ParquetPopUp)
+        convert_data_menu.add_command(label="Convert parquet o CSV", command=Parquet2CsvPopUp)
+
         video_process_menu.add_cascade(
             label="Convert working file type...",
             compound="left",
@@ -1673,30 +1706,11 @@ class App(object):
             image=self.menu_icons["path"]["img"],
             command=MakePathPlotPopUp,
         )
-        video_process_menu.add_command(
-            label="Crop videos",
-            compound="left",
-            image=self.menu_icons["crop"]["img"],
-            command=CropVideoPopUp,
-        )
-        video_process_menu.add_command(
-            label="Crop videos (circles)",
-            compound="left",
-            image=self.menu_icons["circle"]["img"],
-            command=CropVideoCirclesPopUp,
-        )
-        # video_process_menu.add_command(
-        #     label="Crop videos (polygons)",
-        #     compound="left",
-        #     image=self.menu_icons["polygon"]["img"],
-        #     command=CropVideoPolygonsPopUp,
-        # )
-        video_process_menu.add_command(
-            label="Down-sample videos",
-            compound="left",
-            image=self.menu_icons["sample"]["img"],
-            command=DownsampleVideoPopUp,
-        )
+
+        downsample_video_menu = Menu(video_process_menu)
+        downsample_video_menu.add_command(label="Down-sample single video", command=DownsampleSingleVideoPopUp)
+        downsample_video_menu.add_command(label="Down-sample multiple videos", command=DownsampleMultipleVideosPopUp)
+        video_process_menu.add_cascade(label="Down-sample video...", compound="left", image=self.menu_icons["sample"]["img"], menu=downsample_video_menu)
         video_process_menu.add_cascade(
             label="Drop body-parts from tracking data",
             compound="left",
@@ -1733,18 +1747,16 @@ class App(object):
             image=self.menu_icons["calipher"]["img"],
             command=CalculatePixelsPerMMInVideoPopUp,
         )
+
+        video_process_menu.add_command(label="Change video brightness / contrast", compound="left", image=self.menu_icons["brightness"]["img"], command=BrightnessContrastPopUp)
+
+
         video_process_menu.add_command(
             label="Merge frames to video",
             compound="left",
             image=self.menu_icons["merge"]["img"],
             command=MergeFrames2VideoPopUp,
         )
-        video_process_menu.add_command(
-            label="Multi-crop",
-            compound="left",
-            image=self.menu_icons["crop"]["img"],
-            command=MultiCropPopUp,
-        )
         video_process_menu.add_command(
             label="Print classifier info...",
             compound="left",
@@ -1908,9 +1920,7 @@ class App(object):
             wrap="none",
             borderwidth=0,
         )
-        self.txt.insert(
-            INSERT, Defaults.WELCOME_MSG.value + emojis["relaxed"] + "\n" * 2
-        )
+        self.txt.insert(INSERT, Defaults.WELCOME_MSG.value + emojis["relaxed"] + "\n" * 2)
         self.txt.tag_add(TagNames.GREETING.value, "1.0", "3.25")
         y_sb.pack(side=RIGHT, fill=Y)
         self.txt.pack(expand=True, fill="both")
