@@ -590,10 +590,12 @@ class CircularStatisticsMixin(object):
                     np.mean(np.sin(data_2_window)), np.mean(np.cos(data_2_window))
                 )
                 sin_1, sin_2 = np.sin(data_1_window - m1), np.sin(data_2_window - m2)
-                c = np.sum(sin_1 * sin_2) / np.sqrt(
-                    np.sum(sin_1 * sin_1) * np.sum(sin_2 * sin_2)
-                )
-                results[j - 1][i] = np.abs(c)
+                denominator = np.sqrt(np.sum(sin_1 * sin_1) * np.sum(sin_2 * sin_2))
+                numerator = np.sum(sin_1 * sin_2)
+                if denominator == 0:
+                    results[j - 1][i] = 0.0
+                else:
+                    results[j - 1][i] = np.abs(numerator / denominator)
 
         return results.astype(np.float32)
 
@@ -1157,7 +1159,6 @@ class CircularStatisticsMixin(object):
 
         References
         ----------
-
         .. [1] Kanatani, Rangarajan, Hyper least squares fitting of circles and ellipses, `Computational Statistics & Data Analysis`,
                vol. 55, pp. 2197-2208, 2011.
         .. [2] Lapp, Salazar, Champagne. Automated maternal behavior during early life in rodents (AMBER) pipeline, `Scientific Reports`,
