@@ -23,7 +23,7 @@ from simba.utils.checks import (check_float, check_if_dir_exists,
                                 check_if_filepath_list_is_empty,
                                 check_instance, check_str,
                                 check_that_column_exist, check_valid_array,
-                                check_valid_lst)
+                                check_valid_dataframe, check_valid_lst)
 from simba.utils.data import detect_bouts
 from simba.utils.errors import CountError, InvalidInputError
 from simba.utils.printing import SimbaTimer, stdout_success
@@ -104,14 +104,15 @@ class FeatureExtractionSupplemental(FeatureExtractionMixin):
         time-period represented of ``bin_size_s`` seconds. Peak is defined as value is higher than
         in the prior observation (i.e., no future data is involved in comparison).
 
+        .. image:: _static/img/peak_cnt.png
+           :width: 700
+           :align: center
+
         :parameter ndarray data: 1D array of size len(frames) representing feature values.
         :parameter int bin_size_s: The size of the buckets in seconds.
         :parameter int fps: Frame-rate of recorded video.
         :return np.ndarray: Array of size data.shape[0] with peak counts as ratio of len(frames).
 
-        .. image:: _static/img/peak_cnt.png
-           :width: 700
-           :align: center
 
         :example:
         >>> data = np.array([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
@@ -437,9 +438,7 @@ class FeatureExtractionSupplemental(FeatureExtractionMixin):
         )
 
     @staticmethod
-    def spontaneous_alternations(
-        data: pd.DataFrame, arm_names: List[str], center_name: str
-    ) -> Tuple[Dict[Union[str, Tuple[int]], int]]:
+    def spontaneous_alternations(data: pd.DataFrame, arm_names: List[str], center_name: str) -> Tuple[Dict[Union[str, Tuple[int]], int]]:
         """
         Detects spontaneous alternations between a set of user-defined ROIs.
 
@@ -449,10 +448,9 @@ class FeatureExtractionSupplemental(FeatureExtractionMixin):
 
         :param pd.DataFrame data: DataFrame containing shape data where each row represents a frame and each column represents a shape where 0 represents not in ROI and 1 represents inside the ROI
         :param List[str] shape_names: List of column names in the DataFrame corresponding to shape names.
-        :returns Dict[Union[str, Tuple[str], Union[int, float, List[int]]]]:
-            Dict with the following keys and values:
+        :returns Dict[Union[str, Tuple[str], Union[int, float, List[int]]]]: Dict with the following keys and values:
 
-                Dict with the following keys and values:
+        Dict with the following keys and values:
 
         - 'pct_alternation': Percent alternation computed as `(spontaneous alternation cnt / (total number of arm entries - (number of arms - 1))) × 100`
         - 'alternation_cnt': The sliding count of ROI entry sequences of length `len(shape_names)` that are all unique.
@@ -462,7 +460,7 @@ class FeatureExtractionSupplemental(FeatureExtractionMixin):
         - 'same_arm_returns_dict': Dictionary with the keys being the name of the ROI and values are a list of frames when the same-arm-return errors were committed.
         - 'alternate_arm_returns_cnt': Dictionary with the keys being the name of the ROI and values are a list of frames when the alternate-arm-return errors were committed.
         - 'alternations_dict': Dictionary with the keys being unique ROI name tuple sequences of length `len(shape_names)` and values are a list of frames when the sequence was completed.
-        - 'arm_entry_sequence': List of strings representing the sequence of arm entries.
+        - 'arm_entry_sequence': Pandas dataframe with two columns: sequence of arm names entered, the frame the animal entered the arm, the frame that the animal left the arm.
 
         :example:
         >>> data = np.zeros((100, 4), dtype=int)
@@ -512,68 +510,29 @@ class FeatureExtractionSupplemental(FeatureExtractionMixin):
                 alternation_cnt,
             )
 
-        check_instance(
-            source=FeatureExtractionSupplemental.spontaneous_alternations.__name__,
-            instance=data,
-            accepted_types=(pd.DataFrame,),
-        )
-        check_valid_lst(
-            data=arm_names,
-            source=FeatureExtractionSupplemental.spontaneous_alternations.__name__,
-            valid_dtypes=(str,),
-            valid_values=data.columns,
-        )
+        check_instance(source=FeatureExtractionSupplemental.spontaneous_alternations.__name__, instance=data, accepted_types=(pd.DataFrame,))
+        check_valid_lst(data=arm_names,source=FeatureExtractionSupplemental.spontaneous_alternations.__name__,valid_dtypes=(str,),valid_values=data.columns)
         check_str(name="center name", value=center_name, options=data.columns)
-        if center_name in arm_names:
-            InvalidInputError(
-                msg="One ROI is defined both as an arm ans as the center",
-                source=FeatureExtractionSupplemental.spontaneous_alternations.__name__,
-            )
-        if len(list(set(arm_names))) != len(arm_names):
-            InvalidInputError(
-                msg=f"Each arm has to be unique but got {arm_names}",
-                source=FeatureExtractionSupplemental.spontaneous_alternations.__name__,
-            )
+        if center_name in arm_names: InvalidInputError(msg="One ROI is defined both as an arm ans as the center", source=FeatureExtractionSupplemental.spontaneous_alternations.__name__)
+        if len(list(set(arm_names))) != len(arm_names): InvalidInputError(msg=f"Each arm has to be unique but got {arm_names}", source=FeatureExtractionSupplemental.spontaneous_alternations.__name__)
         roi_names = arm_names + [center_name]
         data_df = data[roi_names]
         invalid_vals = list(set(np.unique(data_df.values.flatten())) - {0, 1})
         if len(invalid_vals) > 0:
-            raise CountError(
-                msg=f"When computing spontaneous alternation, ROI fields can only be 0 or 1. Found {invalid_vals}",
-                source=FeatureExtractionSupplemental.spontaneous_alternations.__name__,
-            )
+            raise CountError(msg=f"When computing spontaneous alternation, ROI fields can only be 0 or 1. Found the value(s): {invalid_vals}", source=FeatureExtractionSupplemental.spontaneous_alternations.__name__)
         multiple_rois_frm_idx = np.argwhere(np.sum(data_df.values, axis=1) > 1)
         if multiple_rois_frm_idx.shape[0] > 0:
-            raise CountError(
-                msg=f"When computing spontaneous alternation, animals should only exist in <=1 ROIs in any one frame. In {multiple_rois_frm_idx.shape[0]} frames, the animal exist in more than one ROI.",
-                source=FeatureExtractionSupplemental.spontaneous_alternations.__name__,
-            )
-        bout_df = (
-            detect_bouts(data_df=data_df, target_lst=data_df.columns, fps=1)[
-                ["Event", "Start_frame"]
-            ]
-            .sort_values(["Start_frame"])
-            .reset_index(drop=True)
-        )
-        shifted_ = pd.concat(
-            [bout_df, bout_df.shift(-1).add_suffix("_shifted").reset_index(drop=True)],
-            axis=1,
-        )[["Event", "Event_shifted"]].values
+            raise CountError(msg=f"When computing spontaneous alternation, animals should only exist in <=1 ROIs in any one frame. In {multiple_rois_frm_idx.shape[0]} frames, the animal exist in more than one ROI.", source=FeatureExtractionSupplemental.spontaneous_alternations.__name__)
+        bout_df = (detect_bouts(data_df=data_df, target_lst=data_df.columns, fps=1)[["Event", "Start_frame", "End_frame"]].sort_values(["Start_frame"]).reset_index(drop=True))
+        shifted_ = pd.concat([bout_df, bout_df.shift(-1).add_suffix("_shifted").reset_index(drop=True)], axis=1)[["Event", "Event_shifted"]].values
         unique_counts = [len(list(set(list(x)))) for x in shifted_]
         drop_idx = np.argwhere(np.array(unique_counts) == 1) + 1
         bout_df = bout_df.drop(drop_idx.flatten(), axis=0).reset_index(drop=True)
-        arm_entry_sequence = bout_df[bout_df["Event"] != center_name].values
-        (
-            alternate_arm_returns,
-            same_arm_returns,
-            alternations,
-            alternate_arm_return_cnt,
-            same_arm_return_cnt,
-            alternation_cnt,
-        ) = get_sliding_alternation(data=arm_entry_sequence)
-        pct_alternation = alternation_cnt / (
-            arm_entry_sequence.shape[0] - (len(arm_names) - 1)
-        )
+        arm_entry_sequence = bout_df[bout_df["Event"] != center_name]
+
+        alternate_arm_returns, same_arm_returns, alternations, alternate_arm_return_cnt, same_arm_return_cnt, alternation_cnt = get_sliding_alternation(data=arm_entry_sequence[["Event", "Start_frame"]].values)
+
+        pct_alternation = alternation_cnt / (len(arm_entry_sequence) - (len(arm_names) - 1))
 
         return {
             "pct_alternation": pct_alternation * 100,
@@ -584,7 +543,7 @@ class FeatureExtractionSupplemental(FeatureExtractionMixin):
             "same_arm_returns_dict": same_arm_returns,
             "alternate_arm_returns_dict": alternate_arm_returns,
             "alternations_dict": alternations,
-            "arm_entry_sequence": arm_entry_sequence[:, 0].flatten(),
+            "arm_entry_sequence": arm_entry_sequence,
         }
 
     @staticmethod
@@ -593,7 +552,7 @@ class FeatureExtractionSupplemental(FeatureExtractionMixin):
         Compute the loops detected within a 2-dimensional path.
 
         .. image:: _static/img/find_path_loops.png
-           :width: 500
+           :width: 700
            :align: center
 
         :param np.ndarray data: Nx2 2-dimensional array with the x and y coordinated represented on axis 1.
@@ -637,8 +596,12 @@ class FeatureExtractionSupplemental(FeatureExtractionMixin):
         A value closer to 1.0 indicates that behavior T always precede behavior C. A value closer to 0.0 indicates that behavior T follows behavior C. A value of -1.0 indicates
         that behavior T never precede nor proceed behavior C.
 
+
+        .. seealso::
+           :class:`simba.data_processorsfsttc_calculator.FSTTCCalculator`
+
         :example:
-        >>> df = read_df(file_path='/Users/simon/Desktop/envs/simba/troubleshooting/two_black_animals_14bp/project_folder/csv/targets_inserted/Together_1.csv', file_type='csv')
+        >>> df = pd.DataFrame(np.random.randint(0, 2, (100, 2)), columns=['Attack', 'Sniffing'])
         >>> FeatureExtractionSupplemental.sequential_lag_analysis(data=df, criterion='Attack', target='Sniffing', fps=5, time_window=2.0)
 
         References
@@ -647,11 +610,6 @@ class FeatureExtractionSupplemental(FeatureExtractionMixin):
                2022.
         """
 
-        check_instance(
-            source=FeatureExtractionSupplemental.sequential_lag_analysis.__name__,
-            instance=data,
-            accepted_types=(pd.DataFrame),
-        )
         check_str(
             name=f"{FeatureExtractionSupplemental.sequential_lag_analysis.__name__} criterion",
             value=criterion,
@@ -668,13 +626,15 @@ class FeatureExtractionSupplemental(FeatureExtractionMixin):
         check_float(
             name=f"{FeatureExtractionSupplemental.sequential_lag_analysis.__name__} time-window",
             value=time_window,
-            min_value=0.01,
+            min_value=10e-6,
         )
-        check_that_column_exist(
+        check_valid_dataframe(
             df=data,
-            column_name=[criterion, target],
-            file_name=FeatureExtractionSupplemental.sequential_lag_analysis.__name__,
+            source=f"{FeatureExtractionSupplemental.sequential_lag_analysis.__name__} data",
+            valid_dtypes=(np.float32, np.float64, np.int64, np.int32, float, int),
+            required_fields=[criterion, target],
         )
+
         bouts = detect_bouts(data_df=data, target_lst=[criterion, target], fps=fps)
         if len(bouts) == 0:
             raise CountError(
@@ -716,3 +676,73 @@ class FeatureExtractionSupplemental(FeatureExtractionMixin):
             return 1.0
         else:
             return np.round(preceding_cnt / (preceding_cnt + proceeding_cnt), 3)
+
+    @staticmethod
+    def distance_and_velocity(
+        x: np.ndarray,
+        fps: float,
+        pixels_per_mm: float,
+        centimeters: Optional[bool] = True,
+    ) -> Tuple[float, float]:
+        """
+        Calculate total movement and mean velocity from a sequence of position data.
+
+        :param x: Array containing movement data. For example, created by ``simba.mixins.FeatureExtractionMixin.framewise_euclidean_distance``.
+        :param fps: Frames per second of the data.
+        :param pixels_per_mm: Conversion factor from pixels to millimeters.
+        :param Optional[bool] centimeters: If True, results are returned in centimeters and centimeters per second. Defaults to True.
+        :return Tuple[float, float]: A tuple containing total movement and mean velocity.
+
+        :example:
+        >>> x = np.random.randint(0, 100, (100,))
+        >>> sum_movement, avg_velocity = FeatureExtractionSupplemental.distance_and_velocity(x=x, fps=10, pixels_per_mm=10, centimeters=True)
+        """
+
+        check_valid_array(
+            data=x,
+            source=FeatureExtractionSupplemental.distance_and_velocity.__name__,
+            accepted_ndims=(1, 2),
+            accepted_dtypes=(np.float32, np.float64, np.int32, np.int64, int, float),
+        )
+        check_float(
+            name=f"{FeatureExtractionSupplemental.distance_and_velocity.__name__} fps",
+            value=fps,
+            min_value=1,
+        )
+        check_float(
+            name=f"{FeatureExtractionSupplemental.distance_and_velocity.__name__} pixels_per_mm",
+            value=pixels_per_mm,
+            min_value=10e-6,
+        )
+        if x.ndim == 2:
+            check_valid_array(
+                data=x,
+                source=FeatureExtractionSupplemental.distance_and_velocity.__name__,
+                accepted_axis_1_shape=(2,),
+            )
+            t = np.full((x.shape[0]), 0.0)
+            for i in range(1, x.shape[0]):
+                t[i] = np.linalg.norm(x[i] - x[i - 1])
+            x = np.copy(t) / pixels_per_mm
+        movement = np.sum(x) / pixels_per_mm
+        v = []
+        for i in range(0, x.shape[0], int(fps)):
+            w = x[i : (i + int(fps))]
+            v.append((np.sum(w) / pixels_per_mm) * (1 / (w.shape[0] / int(fps))))
+        if centimeters:
+            v = [vi / 10 for vi in v]
+            movement = movement / 10
+        return movement, np.mean(v)
+
+
+# x = np.random.randint(0, 100, (100, 2))
+# FeatureExtractionSupplemental.distance_and_velocity(
+#     x=x, fps=10, pixels_per_mm=10, centimeters=True
+# )
+# # sum_movement, avg_velocity =
+
+
+# df = read_df(file_path='/Users/simon/Desktop/envs/simba/troubleshooting/two_black_animals_14bp/project_folder/csv/targets_inserted/Together_1.csv', file_type='csv')
+#
+# df = pd.DataFrame(np.random.randint(0, 2, (100, 2)), columns=['Attack', 'Sniffing'])
+# FeatureExtractionSupplemental.sequential_lag_analysis(data=df, criterion='Attack', target='Sniffing', fps=5, time_window=2.0)
