@@ -1,3 +1,4 @@
+import platform
 from typing import Dict, List, Optional, Tuple, Union
 
 import numpy as np
@@ -50,12 +51,12 @@ class ImageMixin(object):
         pass
 
     @staticmethod
-    def brightness_intensity(
-        imgs: List[np.ndarray], ignore_black: Optional[bool] = True
-    ) -> List[float]:
+    def brightness_intensity(imgs: List[np.ndarray], ignore_black: Optional[bool] = True) -> List[float]:
         """
         Compute the average brightness intensity within each image within a list.
 
+        For example, (i) create a list of images containing a light cue ROI, (ii) compute brightness in each image, (iii) perform kmeans on brightness, and get the frames when the light cue is on vs off.
+
         :param List[np.ndarray] imgs: List of images as arrays to calculate average brightness intensity within.
         :param Optional[bool] ignore_black: If True, ignores black pixels. If the images are sliced non-rectangular geometric shapes created by ``slice_shapes_in_img``, then pixels that don't belong to the shape has been masked in black.
         :returns List[float]: List of floats of size len(imgs) with brightness intensities.
@@ -89,17 +90,8 @@ class ImageMixin(object):
     @staticmethod
     def gaussian_blur(img: np.ndarray, kernel_size: Optional[Tuple] = (9, 9)):
         check_if_valid_img(data=img, source=ImageMixin.gaussian_blur.__name__)
-        check_instance(
-            source=ImageMixin.gaussian_blur.__name__,
-            instance=kernel_size,
-            accepted_types=(tuple,),
-        )
-        check_valid_lst(
-            data=list(kernel_size),
-            source=ImageMixin.gaussian_blur.__name__,
-            valid_dtypes=(int,),
-            exact_len=2,
-        )
+        check_instance(source=ImageMixin.gaussian_blur.__name__, instance=kernel_size, accepted_types=(tuple,))
+        check_valid_lst(data=list(kernel_size), source=ImageMixin.gaussian_blur.__name__, valid_dtypes=(int,), exact_len=2,)
         return cv2.GaussianBlur(img, kernel_size, 0)
 
     @staticmethod
@@ -170,13 +162,12 @@ class ImageMixin(object):
             )
 
     @staticmethod
-    def get_contourmatch(
-        img_1: np.ndarray,
-        img_2: np.ndarray,
-        mode: Optional[Literal["all", "exterior"]] = "all",
-        method: Optional[Literal["simple", "none", "l2", "kcos"]] = "simple",
-        canny: Optional[bool] = True,
-    ) -> float:
+    def get_contourmatch(img_1: np.ndarray,
+                         img_2: np.ndarray,
+                         mode: Optional[Literal["all", "exterior"]] = "all",
+                         method: Optional[Literal["simple", "none", "l2", "kcos"]] = "simple",
+                         canny: Optional[bool] = True) -> float:
+
         """
         Calculate contour similarity between two images.
 
@@ -395,13 +386,13 @@ class ImageMixin(object):
                     interior_contours.append(cnts[i])
 
     @staticmethod
-    def orb_matching_similarity_(
-        img_1: np.ndarray,
-        img_2: np.ndarray,
-        method: Literal["knn", "match", "radius"] = "knn",
-        mask: Optional[np.ndarray] = None,
-        threshold: Optional[int] = 0.75,
-    ) -> int:
+    def orb_matching_similarity_(img_1: np.ndarray,
+                                 img_2: np.ndarray,
+                                 method: Literal["knn", "match", "radius"] = "knn",
+                                 mask: Optional[np.ndarray] = None,
+                                 threshold: Optional[int] = 0.75) -> int:
+
+
         """Perform ORB feature matching between two sets of images.
 
         >>> img_1 = cv2.imread('/Users/simon/Desktop/envs/troubleshooting/khan/project_folder/videos/stitched_frames/0.png').astype(np.uint8)
@@ -415,9 +406,7 @@ class ImageMixin(object):
         sliced_matches = None
         if method == "knn":
             matches = cv2.BFMatcher().knnMatch(des1, des2, k=2)
-            sliced_matches = [
-                m for m, n in matches if m.distance < threshold * n.distance
-            ]
+            sliced_matches = [m for m, n in matches if m.distance < threshold * n.distance]
         if method == "match":
             matches = cv2.BFMatcher(cv2.NORM_HAMMING, crossCheck=True).match(des1, des2)
             sliced_matches = [match for match in matches if match.distance <= threshold]
@@ -534,23 +523,29 @@ class ImageMixin(object):
         """
         Convert an image to black and white (binary).
 
+        .. image:: _static/img/img_to_bw.png
+           :width: 600
+           :align: center
+
+        .. note::
+           If converting multiple images from colour to black and white, consider ``simba.mixins.image_mixin.ImageMixin.img_stack_to_bw()``
+
+
         :param np.ndarray img: Input image as a NumPy array.
         :param Optional[int] lower_thresh: Lower threshold value for binary conversion. Pixels below this value become black. Default is 20.
         :param Optional[int] upper_thresh: Upper threshold value for binary conversion. Pixels above this value become white. Default is 250.
         :param Optional[bool] invert: Flag indicating whether to invert the binary image (black becomes white and vice versa). Default is True.
         :return np.ndarray: Binary black and white image.
         """
-        check_if_valid_img(
-            data=img, source=ImageMixin().segment_img_horizontal.__name__
-        )
+        check_if_valid_img(data=img, source=ImageMixin().img_to_bw.__name__)
         check_int(
-            name=f"{ImageMixin().segment_img_horizontal.__name__} lower_thresh",
+            name=f"{ImageMixin().img_to_bw.__name__} lower_thresh",
             value=lower_thresh,
             max_value=255,
             min_value=1,
         )
         check_int(
-            name=f"{ImageMixin().segment_img_horizontal.__name__} upper_thresh",
+            name=f"{ImageMixin().img_to_bw.__name__} upper_thresh",
             value=upper_thresh,
             max_value=255,
             min_value=1,
@@ -562,6 +557,47 @@ class ImageMixin(object):
         else:
             return ~cv2.threshold(img, lower_thresh, upper_thresh, cv2.THRESH_BINARY)[1]
 
+    @staticmethod
+    @jit(nopython=True)
+    def img_stack_to_bw(
+        imgs: np.ndarray, lower_thresh: int, upper_thresh: int, invert: bool
+    ):
+        """
+        Convert a stack of color images into black and white format.
+
+        .. note::
+           If converting a single image, consider ``simba.mixins.image_mixin.ImageMixin.img_to_bw()``
+
+        :param np.ndarray img: 4-dimensional array of color images.
+        :param Optional[int] lower_thresh: Lower threshold value for binary conversion. Pixels below this value become black. Default is 20.
+        :param Optional[int] upper_thresh: Upper threshold value for binary conversion. Pixels above this value become white. Default is 250.
+        :param Optional[bool] invert: Flag indicating whether to invert the binary image (black becomes white and vice versa). Default is True.
+        :return np.ndarray: 4-dimensional array with black and white image.
+
+        :example:
+        >>> imgs = ImageMixin.read_img_batch_from_video(video_path='/Users/simon/Downloads/3A_Mouse_5-choice_MouseTouchBasic_a1.mp4', start_frm=0, end_frm=100)
+        >>> imgs = np.stack(imgs.values(), axis=0)
+        >>> bw_imgs = ImageMixin.img_stack_to_bw(imgs=imgs, upper_thresh=255, lower_thresh=20, invert=False)
+        """
+
+        results = np.full((imgs.shape[:3]), np.nan)
+        for cnt in range(imgs.shape[0]):
+            arr = imgs[cnt]
+            m, n, _ = arr.shape
+            img_mean = np.zeros((m, n))
+            for i in range(m):
+                for j in range(n):
+                    total = 0.0
+                    for k in range(arr.shape[2]):
+                        total += arr[i, j, k]
+                    img_mean[i, j] = total / arr.shape[2]
+            img = np.where(img_mean < lower_thresh, 0, img_mean)
+            img = np.where(img > upper_thresh, 1, img)
+            if invert:
+                img = 1 - img
+            results[cnt] = img
+        return results
+
     @staticmethod
     def segment_img_horizontal(
         img: np.ndarray,
@@ -574,10 +610,19 @@ class ImageMixin(object):
 
         This function segments either the lower, upper, or both lower and upper part of the input image based on the specified percentage.
 
+        .. image:: _static/img/segment_img_horizontal.png
+           :width: 800
+           :align: center
+
         :param np.ndarray img: Input image as a NumPy array.
         :param int pct: Percentage of the image to be segmented. If `lower` is True, it represents the lower part; if False, it represents the upper part.
         :param Optional[bool] lower: Flag indicating whether to segment the lower part (True) or upper part (False) of the image. Default is True.
+        :param Optional[bool] both: If True, **removes** both the upper pct and lower pct and keeps middle part.
         :return np.array: Segmented part of the image.
+
+        :example:
+        >>> img = cv2.imread('/Users/simon/Desktop/test.png')
+        >>> img = ImageMixin.segment_img_horizontal(img=img, pct=10, both=True)
         """
 
         check_if_valid_img(
@@ -597,6 +642,37 @@ class ImageMixin(object):
         else:
             return img[:sliced_height, :]
 
+    @staticmethod
+    @jit(nopython=True, parallel=True)
+    def segment_img_stack_horizontal(
+        imgs: np.ndarray, pct: int, lower: bool, both: bool
+    ) -> np.ndarray:
+        """
+        Segment a horizontal part of all images in stack.
+
+        :example:
+        >>> imgs = ImageMixin.read_img_batch_from_video(video_path='/Users/simon/Downloads/3A_Mouse_5-choice_MouseTouchBasic_a1.mp4', start_frm=0, end_frm=400)
+        >>> imgs = np.stack(imgs.values(), axis=0)
+        >>> sliced_imgs = ImageMixin.segment_img_stack_horizontal(imgs=imgs, pct=50, lower=True, both=False)
+        """
+        results = []
+        for cnt in range(imgs.shape[0]):
+            img = imgs[cnt]
+            sliced_height = int(img.shape[0] * pct / 100)
+            if both:
+                sliced_img = img[sliced_height : img.shape[0] - sliced_height, :]
+            elif lower:
+                sliced_img = img[img.shape[0] - sliced_height :, :]
+            else:
+                sliced_img = img[:sliced_height, :]
+            results.append(sliced_img)
+        stacked_results = np.full(
+            (len(results), results[0].shape[0], results[0].shape[1], 3), np.nan
+        )
+        for i in prange(len(results)):
+            stacked_results[i] = results[i]
+        return results
+
     @staticmethod
     def segment_img_vertical(
         img: np.ndarray,
@@ -609,9 +685,14 @@ class ImageMixin(object):
 
         This function segments either the left, right or both the left and right part of  input image based on the specified percentage.
 
+        .. image:: _static/img/segment_img_vertical.png
+           :width: 800
+           :align: center
+
         :param np.ndarray img: Input image as a NumPy array.
         :param int pct: Percentage of the image to be segmented. If `lower` is True, it represents the lower part; if False, it represents the upper part.
         :param Optional[bool] lower: Flag indicating whether to segment the lower part (True) or upper part (False) of the image. Default is True.
+        :param Optional[bool] both: If True, **removes** both the left pct and right pct and keeps middle part.
         :return np.array: Segmented part of the image.
         """
 
@@ -735,7 +816,9 @@ class ImageMixin(object):
         Pairwise comparison of images in two stacks of equal length using mean squared errors.
 
         .. note::
-           Images has to be in uint8 format.
+           Useful for noting subtle changes, each imgs_2 equals imgs_1 with images shifted by 1. Images has to be in uint8 format.
+           Also see ``img_sliding_mse``.
+
 
         :param np.ndarray imgs_1: First three (non-color) or four (color) dimensional stack of images in array format.
         :param np.ndarray imgs_1: Second three (non-color) or four (color) dimensional stack of images in array format.
@@ -906,7 +989,14 @@ class ImageMixin(object):
     @jit(nopython=True)
     def img_matrix_mse(imgs: np.ndarray) -> np.ndarray:
         """
-        Jitted compute mean squared error matrix table for a stack of images.
+        Compute the mean squared error (MSE) matrix table for a stack of images.
+
+        This function calculates the MSE between each pair of images in the input array
+        and returns a symmetric matrix where each element (i, j) represents the MSE
+        between the i-th and j-th images. Useful for image similarities and anomalities.
+
+        :param np.ndarray imgs: A stack of images represented as a numpy array.
+        :return np.ndarray: The MSE matrix table.
 
         :example:
         >>> imgs = ImageMixin().read_img_batch_from_video(video_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/videos/Together_1.avi', start_frm=0, end_frm=50)
@@ -929,13 +1019,17 @@ class ImageMixin(object):
         """
         Jitted conversion of a 4D stack of color images (RGB format) to grayscale.
 
+        .. image:: _static/img/img_stack_to_greyscale.png
+           :width: 600
+           :align: center
+
         :parameter np.ndarray imgs: A 4D array representing color images. It should have the shape (num_images, height, width, 3) where the last dimension represents the color channels (R, G, B).
         :returns np.ndarray: A 3D array containing the grayscale versions of the input images. The shape of the output array is (num_images, height, width).
 
         :example:
         >>> imgs = ImageMixin().read_img_batch_from_video( video_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/videos/Together_1.avi', start_frm=0, end_frm=100)
         >>> imgs = np.stack(list(imgs.values()))
-        >>> imgs_gray = img_stack_to_greyscale(imgs=imgs)
+        >>> imgs_gray = ImageMixin.img_stack_to_greyscale(imgs=imgs)
         """
         results = np.full((imgs.shape[0], imgs.shape[1], imgs.shape[2]), np.nan).astype(
             np.uint8
@@ -1213,6 +1307,20 @@ class ImageMixin(object):
         )
         return results
 
+    #
+    # def segment_img_horizontal(img: np.ndarray, pct: int, lower: Optional[bool] = True,
+    #                            both: Optional[bool] = False
+
+
+# img = cv2.imread('/Users/simon/Desktop/test.png')
+# img = ImageMixin.segment_img_vertical(img=img, pct=20, both=True)
+# cv2.imshow('sdsdf', img)
+# cv2.waitKey(5000)
+
+
+# bw_img = ImageMixin.img_to_bw(img=img, invert=False)
+# cv2.imshow('sdsdf', bw_img)
+# cv2.waitKey(5000)
 
 # imgs = ImageMixin().read_img_batch_from_video( video_path='/Users/simon/Desktop/envs/troubleshooting/Emergence/project_folder/videos/Example_1.mp4', start_frm=0, end_frm=10)
 # imgs = np.stack(list(imgs.values()))
