@@ -1,7 +1,7 @@
 __author__ = "Simon Nilsson"
 
 import os
-from typing import Dict, List
+from typing import Any, Dict, List, Optional, Union
 
 import cv2
 import numpy as np
@@ -10,12 +10,22 @@ from joblib import Parallel, delayed
 
 from simba.data_processors.movement_calculator import MovementCalculator
 from simba.mixins.config_reader import ConfigReader
+from simba.utils.checks import (
+    check_all_file_names_are_represented_in_video_log,
+    check_if_keys_exist_in_dict, check_valid_lst)
 from simba.utils.enums import Formats
 from simba.utils.errors import NoSpecifiedOutputError
 from simba.utils.lookups import get_color_dict
 from simba.utils.printing import SimbaTimer, stdout_success
 from simba.utils.read_write import get_fn_ext
 
+BG_COLOR = "bg_color"
+HEADER_COLOR = "header_color"
+FONT_THICKNESS = "font_thickness"
+SIZE = "size"
+DATA_ACCURACY = "data_accuracy"
+STYLE_KEYS = [BG_COLOR, HEADER_COLOR, FONT_THICKNESS, SIZE, DATA_ACCURACY]
+
 
 class DataPlotter(ConfigReader):
     """
@@ -37,18 +47,30 @@ class DataPlotter(ConfigReader):
 
     def __init__(
         self,
-        config_path: str,
-        style_attr: Dict,
+        config_path: Union[str, os.PathLike],
+        style_attr: Dict[str, Any],
         body_part_attr: List[List[str]],
         data_paths: List[str],
-        video_setting: bool,
-        frame_setting: bool,
+        video_setting: Optional[bool] = True,
+        frame_setting: Optional[bool] = True,
     ):
-        super().__init__(config_path=config_path)
+
         if (not video_setting) and (not frame_setting):
             raise NoSpecifiedOutputError(
                 msg="SIMBA ERROR: Please choose to create video and/or frames data plots. SimBA found that you ticked neither video and/or frames"
             )
+        check_valid_lst(
+            data=data_paths, source=self.__class__.__name__, valid_dtypes=(str,)
+        )
+        check_valid_lst(
+            data=body_part_attr, source=self.__class__.__name__, valid_dtypes=(list,)
+        )
+        for i in body_part_attr:
+            check_valid_lst(
+                data=i, source=self.__class__.__name__, valid_dtypes=(str,), exact_len=2
+            )
+        check_if_keys_exist_in_dict(data=style_attr, key=STYLE_KEYS)
+        ConfigReader.__init__(self, config_path=config_path)
         self.video_setting, self.frame_setting = video_setting, frame_setting
         self.files_found, self.style_attr, self.body_part_attr = (
             data_paths,
@@ -175,6 +197,9 @@ class DataPlotter(ConfigReader):
                 )
             return img
 
+        check_all_file_names_are_represented_in_video_log(
+            video_info_df=self.video_info_df, data_paths=self.files_found
+        )
         for file_cnt, file_path in enumerate(self.files_found):
             video_timer = SimbaTimer(start=True)
             _, video_name, _ = get_fn_ext(file_path)
@@ -184,7 +209,7 @@ class DataPlotter(ConfigReader):
             if self.video_setting:
                 self.fourcc = cv2.VideoWriter_fourcc(*Formats.MP4_CODEC.value)
                 self.video_save_path = os.path.join(
-                    self.data_table_path, video_name + ".mp4"
+                    self.data_table_path, f"{video_name}.mp4"
                 )
                 self.writer = cv2.VideoWriter(
                     self.video_save_path, self.fourcc, self.fps, self.style_attr["size"]
@@ -248,11 +273,11 @@ class DataPlotter(ConfigReader):
 
 
 # style_attr = {'bg_color': 'White', 'header_color': 'Black', 'font_thickness': 1, 'size': (640, 480), 'data_accuracy': 2}
-# body_part_attr = [['Ear_left_1', 'Grey'], ['Ear_right_2', 'Red']]
-# data_paths = ['/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/csv/machine_results/Together_1.csv']
+# body_part_attr = [['Ear_left_1', 'Green'], ['Ear_right_2', 'Red']]
+# data_paths = ['/Users/simon/Desktop/envs/simba/troubleshooting/beepboop174/project_folder/csv/machine_results/Trial    10.csv']
 #
 #
-# test = DataPlotter(config_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini',
+# test = DataPlotter(config_path='/Users/simon/Desktop/envs/simba/troubleshooting/beepboop174/project_folder/project_config.ini',
 #                    style_attr=style_attr,
 #                    body_part_attr=body_part_attr,
 #                    data_paths=data_paths,
