@@ -44,22 +44,22 @@ class PlotSklearnResultsSingleCore(ConfigReader, TrainModelMixin, PlottingMixin)
     :param Optional[dict] text_settings:
     :param str video_file_path: path to video file to create classification visualizations for.
 
-    Examples
-    ----------
+    :example:
     >>> clf_plotter = PlotSklearnResultsSingleCore(config_path='MyProjectConfig', video_setting=True, frame_setting=False, rotate=False, video_file_path='VideoPath')
     >>> clf_plotter.run()
     """
 
-    def __init__(
-        self,
-        config_path: str,
-        video_setting: bool,
-        frame_setting: bool,
-        text_settings: Optional[Union[Dict[str, float], bool]] = False,
-        video_file_path: Optional[List] = None,
-        rotate: bool = False,
-        print_timers: bool = True,
-    ):
+    def __init__(self,
+                 config_path: Union[str, os.PathLike],
+                 video_setting: Optional[bool] = True,
+                 frame_setting: Optional[bool] = False,
+                 text_settings: Optional[Union[Dict[str, float], bool]] = False,
+                 video_file_path: Optional[List] = None,
+                 rotate: bool = False,
+                 print_timers: bool = True):
+
+
+
         ConfigReader.__init__(self, config_path=config_path)
         TrainModelMixin.__init__(self)
         PlottingMixin.__init__(self)
