@@ -19,7 +19,8 @@ from simba.utils.data import create_color_palette
 from simba.utils.enums import ConfigKey, Dtypes, Formats, TagNames
 from simba.utils.errors import NoSpecifiedOutputError
 from simba.utils.printing import SimbaTimer, log_event, stdout_success
-from simba.utils.read_write import (concatenate_videos_in_folder, get_fn_ext,
+from simba.utils.read_write import (concatenate_videos_in_folder,
+                                    find_core_cnt, get_fn_ext,
                                     get_video_meta_data, read_config_entry,
                                     read_df)
 
@@ -176,7 +177,7 @@ class PlotSklearnResultsMultiProcess(ConfigReader, TrainModelMixin, PlottingMixi
     Plot classification results on videos. Results are stored in the
     `project_folder/frames/output/sklearn_results` directory of the SimBA project.
 
-    .. seelalso::
+    .. seealso::
        `Tutorial <https://github.com/sgoldenlab/simba/blob/master/docs/tutorial.md#step-10-sklearn-visualization__.
         For non-multiptocess class, see :meth:`simba.plotting.plot_clf_results.PlotSklearnResultsSingleCore`.
 
@@ -184,7 +185,6 @@ class PlotSklearnResultsMultiProcess(ConfigReader, TrainModelMixin, PlottingMixi
        :width: 600
        :align: center
 
-
     :param str config_path: path to SimBA project config file in Configparser format
     :param bool rotate: If True, the output video will be rotated 90 degrees from the input.
     :param bool video_setting: If True, SimBA will create compressed videos.
@@ -192,24 +192,31 @@ class PlotSklearnResultsMultiProcess(ConfigReader, TrainModelMixin, PlottingMixi
     :param str video_file_path: path to video file to create classification visualizations for.
     :param int cores: Number of cores to use
 
-
-    Examples
-    ----------
+    :example:
     >>> clf_plotter = PlotSklearnResultsMultiProcess(config_path='MyProjectConfig', video_setting=True, frame_setting=False, rotate=False, video_file_path='VideoPath', cores=5)
     >>> clf_plotter.run()
     """
 
     def __init__(
         self,
-        config_path: str,
-        video_setting: bool,
-        frame_setting: bool,
-        cores: int,
+        config_path: Union[str, os.PathLike],
+        video_setting: Optional[bool] = True,
+        frame_setting: Optional[bool] = True,
+        cores: Optional[int] = -1,
         video_file_path: Optional[str] = None,
         text_settings: Optional[Union[Dict[str, float], bool]] = False,
         rotate: Optional[bool] = False,
         print_timers: Optional[bool] = True,
     ):
+
+        if (not video_setting) and (not frame_setting):
+            raise NoSpecifiedOutputError(
+                msg="SIMBA ERROR: Please choose to create a video and/or frames. SimBA found that you ticked neither video and/or frames",
+                source=self.__class__.__name__,
+            )
+        check_int(name=f"{self.__class__.__name__} cores", value=cores, min_value=-1)
+        if cores == -1:
+            cores = find_core_cnt()[0]
         ConfigReader.__init__(self, config_path=config_path)
         TrainModelMixin.__init__(self)
         PlottingMixin.__init__(self)
@@ -222,11 +229,6 @@ class PlotSklearnResultsMultiProcess(ConfigReader, TrainModelMixin, PlottingMixi
         if platform.system() == "Darwin":
             multiprocessing.set_start_method("spawn", force=True)
 
-        if (not video_setting) and (not frame_setting):
-            raise NoSpecifiedOutputError(
-                msg="SIMBA ERROR: Please choose to create a video and/or frames. SimBA found that you ticked neither video and/or frames",
-                source=self.__class__.__name__,
-            )
         self.video_file_path, self.print_timers, self.text_settings = (
             video_file_path,
             print_timers,
@@ -238,8 +240,6 @@ class PlotSklearnResultsMultiProcess(ConfigReader, TrainModelMixin, PlottingMixi
             cores,
             rotate,
         )
-        if video_file_path is not None:
-            check_file_exist_and_readable(os.path.join(self.video_dir, video_file_path))
         if not os.path.exists(self.sklearn_plot_dir):
             os.makedirs(self.sklearn_plot_dir)
         self.pose_threshold = read_config_entry(
@@ -255,6 +255,10 @@ class PlotSklearnResultsMultiProcess(ConfigReader, TrainModelMixin, PlottingMixi
         )
         self.fourcc = cv2.VideoWriter_fourcc(*Formats.MP4_CODEC.value)
 
+        if video_file_path is not None:
+            check_file_exist_and_readable(os.path.join(self.video_dir, video_file_path))
+            self.data_files = video_file_path
+
     def __get_print_settings(self):
         self.text_attr = {}
         if self.text_settings is False:
@@ -417,5 +421,14 @@ class PlotSklearnResultsMultiProcess(ConfigReader, TrainModelMixin, PlottingMixi
             )
 
 
+# clf_plotter = PlotSklearnResultsMultiProcess(config_path='/Users/simon/Desktop/envs/simba/troubleshooting/beepboop174/project_folder/project_config.ini',
+#                                              video_setting=True,
+#                                              frame_setting=False,
+#                                              rotate=False,
+#                                              video_file_path='Trial    10.mp4',
+#                                              cores=5)
+# clf_plotter.run()
+
+
 # clf_plotter = PlotSklearnResultsMultiProcess(config_path='/Users/simon/Desktop/envs/troubleshooting/DLC_2_Black_animals/project_folder/project_config.ini', video_setting=True, frame_setting=False, rotate=False, video_file_path='Together_1.avi', cores=5)
 # clf_plotter.run()
