@@ -2,6 +2,7 @@ __author__ = "Simon Nilsson"
 
 import itertools
 import os
+from typing import Optional, Union
 
 import cv2
 import numpy as np
@@ -9,23 +10,29 @@ import pandas as pd
 
 import simba
 from simba.mixins.config_reader import ConfigReader
-from simba.utils.checks import check_file_exist_and_readable
+from simba.utils.checks import (check_file_exist_and_readable,
+                                check_if_df_field_is_boolean,
+                                check_if_dir_exists, check_instance, check_int,
+                                check_str, check_that_column_exist)
 from simba.utils.enums import Paths
 from simba.utils.printing import SimbaTimer, stdout_success
 from simba.utils.warnings import ShapWarning
 
+SIMBA_DIR = os.path.dirname(simba.__file__)
+
 
 class ShapAggregateStatisticsVisualizer(ConfigReader):
     """
     Calculate aggregate (binned) SHAP value statistics where individual bins represent reaulated features.
-    Also creates line chart visualizations reprsenting aggregations of behavior-present SHAP values.
+    and create line chart visualizations reprsenting aggregations of behavior-present SHAP values.
 
     .. note::
        `Tutorial <https://github.com/sgoldenlab/simba/blob/master/docs/SHAP.md#step-3-interpreting-the-shap-value-ouput-generated-by-simba>`__.
        `Example output <https://github.com/sgoldenlab/simba/blob/master/images/example_shap_graph.png>`__.
-        .. image:: _static/img/shap.png
-           :width: 400
-           :align: center
+
+    .. image:: _static/img/shap.png
+       :width: 600
+       :align: center
 
 
     :parameter str config_path: Path to SimBA project config file in Configparser format
@@ -34,28 +41,42 @@ class ShapAggregateStatisticsVisualizer(ConfigReader):
     :param float shap_baseline_value: SHAP expected value (computed by ``simba.train_model_functions.create_shap_log``).
     :param str save_path: Directory where to store the results
 
-
-
-
     :example:
     >>> _ = ShapAggregateStatisticsVisualizer(config_path='SimBAConfigFilePath', classifier_name='Attack', shap_df='tests/test_data/test_shap/data/test_shap.csv', shap_baseline_value=4, save_path='SaveDirectory')
     """
 
-    def __init__(
-        self,
-        config_path: str,
-        classifier_name: str,
-        shap_df: pd.DataFrame,
-        shap_baseline_value: int,
-        save_path: str,
-    ):
+    def __init__(self,
+                 config_path: Union[str, os.PathLike],
+                 shap_df: pd.DataFrame,
+                 classifier_name: str,
+                 shap_baseline_value: int,
+                 visualization: Optional[bool] = True,
+                 save_path: Optional[Union[str, os.PathLike]] = None):
+
+        check_file_exist_and_readable(file_path=config_path)
+        check_instance(
+            source=f"{self.__class__.__name__} shap_df",
+            instance=shap_df,
+            accepted_types=(pd.DataFrame,),
+        )
+        check_str(
+            name=f"{self.__class__.__name__} classifier_name", value=classifier_name
+        )
+        check_that_column_exist(
+            df=shap_df, column_name=classifier_name, file_name="shap dataframe"
+        )
+        check_if_df_field_is_boolean(df=shap_df, field=classifier_name)
+        check_int(
+            name=f"{self.__class__.__name__} shap_baseline_value",
+            value=shap_baseline_value,
+            max_value=100,
+            min_value=0,
+        )
+
         ConfigReader.__init__(self, config_path=config_path)
         if (self.pose_setting != "14") and (self.pose_setting != "16"):
-            ShapWarning(
-                "SHAP visualizations/aggregate stats skipped (only viable for projects with two animals and default 7 or 8 body-parts per animal) ..."
-            )
+            ShapWarning(msg="SHAP visualizations/aggregate stats skipped (only viable for projects with two animals and default 7 or 8 body-parts per animal) ...", source=self.__class__.__name__)
         else:
-            self.save_path = save_path
             self.classifier_name, self.shap_df, self.shap_baseline_value = (
                 classifier_name,
                 shap_df,
@@ -65,81 +86,22 @@ class ShapAggregateStatisticsVisualizer(ConfigReader):
                 os.makedirs(self.shap_logs_path)
             self.img_save_path = os.path.join(
                 self.shap_logs_path,
-                "SHAP_summary_line_graph_{}_{}.png".format(
-                    self.classifier_name, self.datetime
-                ),
+                f"SHAP_summary_line_graph_{self.classifier_name}_{self.datetime}.png",
             )
-            simba_dir = os.path.dirname(simba.__file__)
             feature_categories_csv_path = os.path.join(
-                simba_dir, Paths.SIMBA_SHAP_CATEGORIES_PATH.value
+                SIMBA_DIR, Paths.SIMBA_SHAP_CATEGORIES_PATH.value
             )
-            shap_img_path = os.path.join(simba_dir, Paths.SIMBA_SHAP_IMG_PATH.value)
             check_file_exist_and_readable(file_path=feature_categories_csv_path)
-            self.scale_img_dict = {
-                "baseline_scale": os.path.join(shap_img_path, "baseline_scale.jpg"),
-                "small_arrow": os.path.join(shap_img_path, "down_arrow.jpg"),
-                "side_scale": os.path.join(shap_img_path, "side_scale.jpg"),
-                "color_bar": os.path.join(shap_img_path, "color_bar.jpg"),
-            }
-            self.category_img_dict = {
-                "Animal distances": {
-                    "icon": os.path.join(shap_img_path, "animal_distances.jpg")
-                },
-                "Intruder movement": {
-                    "icon": os.path.join(shap_img_path, "intruder_movement.jpg")
-                },
-                "Resident+intruder movement": {
-                    "icon": os.path.join(
-                        shap_img_path, "resident_intruder_movement.jpg"
-                    )
-                },
-                "Resident movement": {
-                    "icon": os.path.join(shap_img_path, "resident_movement.jpg")
-                },
-                "Intruder shape": {
-                    "icon": os.path.join(shap_img_path, "intruder_shape.jpg")
-                },
-                "Resident+intruder shape": {
-                    "icon": os.path.join(shap_img_path, "resident_intruder_shape.jpg")
-                },
-                "Resident shape": {
-                    "icon": os.path.join(shap_img_path, "resident_shape.jpg")
-                },
-            }
-            for k, v in self.scale_img_dict.items():
-                check_file_exist_and_readable(file_path=v)
-            for k, v in self.category_img_dict.items():
-                check_file_exist_and_readable(file_path=v["icon"])
-            self.positive_arrow_colors = [
-                (253, 141, 60),
-                (252, 78, 42),
-                (227, 26, 28),
-                (189, 0, 38),
-                (128, 0, 38),
-            ]
-            self.negative_arrow_colors = [
-                (65, 182, 196),
-                (29, 145, 192),
-                (34, 94, 168),
-                (37, 52, 148),
-                (8, 29, 88),
-            ]
-            self.ranges_lst = [
-                list(range(0, 20)),
-                list(range(20, 40)),
-                list(range(40, 60)),
-                list(range(60, 80)),
-                list(range(80, 101)),
-            ]
             self.feature_categories_df = pd.read_csv(
                 feature_categories_csv_path, header=[0, 1]
             )
             self.unique_feature_category_names, self.unique_time_bin_names = set(
                 list(self.feature_categories_df.columns.levels[0])
             ), set(list(self.feature_categories_df.columns.levels[1]))
-            self.__calculate_agg_shap_scores()
-            self.__create_base_shap_img()
-            self.__insert_data_in_img()
+            self.__run()
+            if visualization:
+                self.__create_base_shap_img()
+                self.__insert_data_in_img()
 
     def __save_aggregate_scores(self):
         """
@@ -156,22 +118,18 @@ class ShapAggregateStatisticsVisualizer(ConfigReader):
             self.df_save_path
         )
 
-    def __calculate_agg_shap_scores(self):
-        """
-        Private helper to aggregate SHAP values into bins of related features.
-        """
+    def __run(self):
         self.agg_stats_timer = SimbaTimer(start=True)
         for clf_state, clf_state_name in zip(range(2), ["ABSENT", "PRESENT"]):
             self.results = {}
             self.df_save_path = os.path.join(
                 self.shap_logs_path,
-                "SHAP_summary_{}_{}_{}.csv".format(
-                    self.classifier_name, clf_state_name, self.datetime
-                ),
+                f"SHAP_summary_{self.classifier_name}_{clf_state_name}_{self.datetime}.csv",
             )
             shap_clf_sliced = self.shap_df[
                 self.shap_df[self.classifier_name] == clf_state
             ]
+
             for feature_category, feature_time_bin in itertools.product(
                 self.unique_feature_category_names, self.unique_time_bin_names
             ):
@@ -193,7 +151,6 @@ class ShapAggregateStatisticsVisualizer(ConfigReader):
             self.__save_aggregate_scores()
         self.agg_stats_timer.stop_timer()
         self.visualization_timer = SimbaTimer(start=True)
-
         stdout_success(
             msg=f"Aggregate SHAP statistics saved in {self.shap_logs_path} directory",
             elapsed_time=self.agg_stats_timer.elapsed_time_str,
@@ -201,9 +158,64 @@ class ShapAggregateStatisticsVisualizer(ConfigReader):
 
     def __create_base_shap_img(self):
         """
-        Private helper to create the base (axes, icons ticks etc) of the aggregate shap value visualization.
+        Private helper to create the base (axes, icons ticks etc.) of the aggregate shap value visualization.
         """
-
+        shap_img_path = os.path.join(SIMBA_DIR, Paths.SIMBA_SHAP_IMG_PATH.value)
+        check_if_dir_exists(in_dir=shap_img_path)
+        self.scale_img_dict = {
+            "baseline_scale": os.path.join(shap_img_path, "baseline_scale.jpg"),
+            "small_arrow": os.path.join(shap_img_path, "down_arrow.jpg"),
+            "side_scale": os.path.join(shap_img_path, "side_scale.jpg"),
+            "color_bar": os.path.join(shap_img_path, "color_bar.jpg"),
+        }
+        self.category_img_dict = {
+            "Animal distances": {
+                "icon": os.path.join(shap_img_path, "animal_distances.jpg")
+            },
+            "Intruder movement": {
+                "icon": os.path.join(shap_img_path, "intruder_movement.jpg")
+            },
+            "Resident+intruder movement": {
+                "icon": os.path.join(shap_img_path, "resident_intruder_movement.jpg")
+            },
+            "Resident movement": {
+                "icon": os.path.join(shap_img_path, "resident_movement.jpg")
+            },
+            "Intruder shape": {
+                "icon": os.path.join(shap_img_path, "intruder_shape.jpg")
+            },
+            "Resident+intruder shape": {
+                "icon": os.path.join(shap_img_path, "resident_intruder_shape.jpg")
+            },
+            "Resident shape": {
+                "icon": os.path.join(shap_img_path, "resident_shape.jpg")
+            },
+        }
+        for k, v in self.scale_img_dict.items():
+            check_file_exist_and_readable(file_path=v)
+        for k, v in self.category_img_dict.items():
+            check_file_exist_and_readable(file_path=v["icon"])
+        self.positive_arrow_colors = [
+            (253, 141, 60),
+            (252, 78, 42),
+            (227, 26, 28),
+            (189, 0, 38),
+            (128, 0, 38),
+        ]
+        self.negative_arrow_colors = [
+            (65, 182, 196),
+            (29, 145, 192),
+            (34, 94, 168),
+            (37, 52, 148),
+            (8, 29, 88),
+        ]
+        self.ranges_lst = [
+            list(range(0, 20)),
+            list(range(20, 40)),
+            list(range(40, 60)),
+            list(range(60, 80)),
+            list(range(80, 101)),
+        ]
         self.img = 255 * np.ones([1680, 1680, 3], dtype=np.uint8)
         self.baseline_scale_img = cv2.imread(self.scale_img_dict["baseline_scale"])
         self.baseline_scale_top_left = (100, 800)
@@ -312,9 +324,7 @@ class ShapAggregateStatisticsVisualizer(ConfigReader):
         data_df = pd.read_csv(
             os.path.join(
                 self.shap_logs_path,
-                "SHAP_summary_{}_{}_{}.csv".format(
-                    self.classifier_name, "PRESENT", self.datetime
-                ),
+                f"SHAP_summary_{self.classifier_name}_PRESENT_{self.datetime}.csv",
             ),
             index_col=0,
         )
@@ -437,3 +447,11 @@ class ShapAggregateStatisticsVisualizer(ConfigReader):
 #                                          shap_df=shap_df,
 #                                          shap_baseline_value=40,
 #                                          save_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/logs/shap')
+
+# shap_df = pd.read_csv('/Users/simon/Desktop/envs/simba/simba/tests/data/sample_data/shap_test.csv', index_col=0)
+# test = ShapAggregateStatisticsVisualizer(config_path='/Users/simon/Desktop/envs/NG_Unsupervised/project_folder/project_config.ini',
+#                                          classifier_name='target',
+#                                          shap_df=shap_df,
+#                                          shap_baseline_value=40,
+#                                          save_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/logs/shap')
+#
