@@ -6,7 +6,7 @@ warnings.filterwarnings("ignore", category=FutureWarning)
 warnings.filterwarnings("ignore", category=DeprecationWarning)
 import os
 import warnings
-from typing import Any, Dict
+from typing import Any, Dict, Optional, Union
 
 import cv2
 import matplotlib.pyplot as plt
@@ -15,11 +15,13 @@ import numpy as np
 from simba.mixins.config_reader import ConfigReader
 from simba.mixins.plotting_mixin import PlottingMixin
 from simba.mixins.train_model_mixin import TrainModelMixin
+from simba.utils.checks import (check_file_exist_and_readable, check_float,
+                                check_if_keys_exist_in_dict, check_int)
 from simba.utils.data import plug_holes_shortest_bout
-from simba.utils.enums import TagNames
+from simba.utils.enums import Formats, TagNames, TextOptions
 from simba.utils.printing import log_event, stdout_success
 from simba.utils.read_write import (get_fn_ext, get_video_meta_data, read_df,
-                                    write_df)
+                                    read_pickle, write_df)
 
 plt.interactive(True)
 plt.ioff()
@@ -58,14 +60,15 @@ class ValidateModelOneVideo(ConfigReader, PlottingMixin, TrainModelMixin):
 
     def __init__(
         self,
-        config_path: str,
-        feature_file_path: str,
-        model_path: str,
-        discrimination_threshold: float,
-        shortest_bout: int,
-        create_gantt: str,
+        config_path: Union[str, os.PathLike],
+        feature_file_path: Union[str, os.PathLike],
+        model_path: Union[str, os.PathLike],
         settings: Dict[str, Any],
+        discrimination_threshold: Optional[float] = 0.0,
+        shortest_bout: Optional[int] = 0.0,
+        create_gantt: Optional[Union[None, int]] = None,
     ):
+
         ConfigReader.__init__(self, config_path=config_path)
         PlottingMixin.__init__(self)
         TrainModelMixin.__init__(self)
@@ -74,120 +77,140 @@ class ValidateModelOneVideo(ConfigReader, PlottingMixin, TrainModelMixin):
             log_type=TagNames.CLASS_INIT.value,
             msg=self.create_log_msg_from_init_args(locals=locals()),
         )
+        check_int(
+            name=f"{self.__class__.__name__} shortest_bout",
+            value=shortest_bout,
+            min_value=0,
+        )
+        check_float(
+            name=f"{self.__class__.__name__} discrimination_threshold",
+            value=discrimination_threshold,
+            min_value=0,
+            max_value=1.0,
+        )
+        check_int(
+            name=f"{self.__class__.__name__} cores", value=shortest_bout, min_value=-1
+        )
+        check_int(
+            name=f"{self.__class__.__name__} cores", value=shortest_bout, min_value=-1
+        )
+        check_file_exist_and_readable(file_path=feature_file_path)
+        check_file_exist_and_readable(file_path=model_path)
+        check_if_keys_exist_in_dict(
+            data=settings,
+            key=["pose", "animal_names", "styles"],
+            name=f"{self.__class__.__name__} settings",
+        )
+        if create_gantt is not None:
+            check_int(
+                name=f"{self.__class__.__name__} create gantt",
+                value=create_gantt,
+                max_value=2,
+                min_value=1,
+            )
         _, self.feature_filename, ext = get_fn_ext(feature_file_path)
+        if not os.path.exists(self.single_validation_video_save_dir):
+            os.makedirs(self.single_validation_video_save_dir)
+        _, _, self.fps = self.read_video_info(video_name=self.feature_filename)
+        self.video_path = self.find_video_of_file(self.video_dir, self.feature_filename)
+        self.video_meta_data = get_video_meta_data(video_path=self.video_path)
+        self.clf_name, self.feature_file_path = (
+            os.path.basename(model_path).replace(".sav", ""),
+            feature_file_path,
+        )
+        self.vid_output_path = os.path.join(
+            self.single_validation_video_save_dir,
+            f"{self.feature_filename} {self.clf_name}.mp4",
+        )
+        self.clf_data_save_path = os.path.join(
+            self.clf_data_validation_dir, self.feature_filename + ".csv"
+        )
+        self.clf = read_pickle(data_path=model_path, verbose=True)
+        self.data_df = read_df(feature_file_path, self.file_type)
+        self.x_df = self.drop_bp_cords(df=self.data_df)
         (
             self.discrimination_threshold,
             self.shortest_bout,
             self.create_gantt,
             self.settings,
         ) = (float(discrimination_threshold), shortest_bout, create_gantt, settings)
-        if not os.path.exists(self.single_validation_video_save_dir):
-            os.makedirs(self.single_validation_video_save_dir)
-        _, _, self.fps = self.read_video_info(video_name=self.feature_filename)
-        self.clf_name, self.feature_file_path = (
-            os.path.basename(model_path).replace(".sav", ""),
-            feature_file_path,
-        )
-        self.video_path = self.find_video_of_file(self.video_dir, self.feature_filename)
-        self.video_meta_data = get_video_meta_data(video_path=self.video_path)
-        self.vid_output_path = os.path.join(
-            self.single_validation_video_save_dir,
-            f"{self.feature_filename} {self.clf_name}.avi",
-        )
-        self.clf_data_save_path = os.path.join(
-            self.clf_data_validation_dir, self.feature_filename + ".csv"
-        )
-        self.clf = read_df(file_path=model_path, file_type="pickle")
-        self.in_df = read_df(feature_file_path, self.file_type)
 
-    def __run_clf(self):
-        self.data_df = self.drop_bp_cords(df=self.in_df)
+    def run(self):
         self.prob_col_name = f"Probability_{self.clf_name}"
         self.data_df[self.prob_col_name] = self.clf_predict_proba(
             clf=self.clf,
-            x_df=self.data_df,
+            x_df=self.x_df,
             model_name=self.clf_name,
             data_path=self.feature_file_path,
         )
         self.data_df[self.clf_name] = np.where(
             self.data_df[self.prob_col_name] > self.discrimination_threshold, 1, 0
         )
-
-    def __plug_bouts(self):
-        self.data_df = plug_holes_shortest_bout(
-            data_df=self.data_df,
-            clf_name=self.clf_name,
-            fps=self.fps,
-            shortest_bout=self.shortest_bout,
-        )
-
-    def __save(self):
-        write_df(
+        if self.shortest_bout > 1:
+            self.data_df = plug_holes_shortest_bout(
+                data_df=self.data_df,
+                clf_name=self.clf_name,
+                fps=self.fps,
+                shortest_bout=self.shortest_bout,
+            )
+        _ = write_df(
             df=self.data_df, file_type=self.file_type, save_path=self.clf_data_save_path
         )
         print(f"Predictions created for video {self.feature_filename}...")
-
-    def __create_video(self):
         cap = cv2.VideoCapture(self.video_path)
-        fourcc = cv2.VideoWriter_fourcc(*"mp4v")
-        if self.create_gantt == "None":
-            writer = cv2.VideoWriter(
-                self.vid_output_path,
-                fourcc,
-                self.fps,
-                (self.video_meta_data["width"], self.video_meta_data["height"]),
-            )
-        else:
+        fourcc, font = cv2.VideoWriter_fourcc(*"mp4v"), cv2.FONT_HERSHEY_COMPLEX
+        if self.create_gantt is not None:
             self.bouts_df = self.get_bouts_for_gantt(
                 data_df=self.data_df, clf_name=self.clf_name, fps=self.fps
             )
-            self.gantt_img = self.create_gantt_img(
+            self.final_gantt_img = self.create_gantt_img(
                 self.bouts_df,
                 self.clf_name,
                 len(self.data_df),
                 self.fps,
                 "Behavior gantt chart (entire session)",
             )
-            self.gantt_img = self.resize_gantt(
-                self.gantt_img, self.video_meta_data["height"]
+            self.final_gantt_img = self.resize_gantt(
+                self.final_gantt_img, self.video_meta_data["height"]
+            )
+            video_size = (
+                int(self.video_meta_data["width"] + self.final_gantt_img.shape[1]),
+                int(self.video_meta_data["height"]),
             )
             writer = cv2.VideoWriter(
-                self.vid_output_path,
-                fourcc,
-                self.fps,
-                (
-                    int(self.video_meta_data["width"] + self.gantt_img.shape[1]),
-                    int(self.video_meta_data["height"]),
-                ),
+                self.vid_output_path, fourcc, self.video_meta_data["fps"], video_size
+            )
+        else:
+            video_size = (
+                int(self.video_meta_data["width"]),
+                int(self.video_meta_data["height"]),
+            )
+            writer = cv2.VideoWriter(
+                self.vid_output_path, fourcc, self.video_meta_data["fps"], video_size
             )
-
         if self.settings["styles"] is None:
             self.settings["styles"] = {}
-            space_scaler, radius_scaler, resolution_scaler, font_scaler = (
-                60,
-                20,
-                1500,
-                1.5,
-            )
             max_dim = max(self.video_meta_data["width"], self.video_meta_data["height"])
             self.settings["styles"]["circle size"] = int(
-                radius_scaler / (resolution_scaler / max_dim)
+                TextOptions.RADIUS_SCALER.value
+                / (TextOptions.RESOLUTION_SCALER.value / max_dim)
             )
             self.settings["styles"]["font size"] = float(
-                font_scaler / (resolution_scaler / max_dim)
+                TextOptions.FONT_SCALER.value
+                / (TextOptions.RESOLUTION_SCALER.value / max_dim)
             )
             self.settings["styles"]["space_scale"] = int(
-                space_scaler / (resolution_scaler / max_dim)
+                TextOptions.SPACE_SCALER.value
+                / (TextOptions.RESOLUTION_SCALER.value / max_dim)
             )
 
+        self.data_df = self.data_df.head(
+            min(len(self.data_df), self.video_meta_data["frame_count"])
+        )
         frm_cnt, clf_frm_cnt = 0, 0
-        while cap.isOpened():
+        print("Creating video...")
+        while (cap.isOpened()) and (frm_cnt < len(self.data_df)):
             ret, frame = cap.read()
-            if frame is None:
-                print(
-                    "SIMBA WARNING: Some frames appear to be missing in the video vs the data file."
-                )
-                break
             clf_val = int(self.data_df.loc[frm_cnt, self.clf_name])
             clf_frm_cnt += clf_val
             if self.settings["pose"]:
@@ -200,8 +223,8 @@ class ValidateModelOneVideo(ConfigReader, PlottingMixin, TrainModelMixin):
                             animal_data["Y_bps"][bp],
                         )
                         animal_cords = tuple(
-                            self.in_df.loc[
-                                self.in_df.index[frm_cnt], [x_header, y_header]
+                            self.data_df.loc[
+                                self.data_df.index[frm_cnt], [x_header, y_header]
                             ]
                         )
                         cv2.circle(
@@ -220,7 +243,9 @@ class ValidateModelOneVideo(ConfigReader, PlottingMixin, TrainModelMixin):
                         animal_data["Y_bps"][0],
                     )
                     animal_cords = tuple(
-                        self.in_df.loc[self.in_df.index[frm_cnt], [x_header, y_header]]
+                        self.data_df.loc[
+                            self.data_df.index[frm_cnt], [x_header, y_header]
+                        ]
                     )
                     cv2.putText(
                         frame,
@@ -235,30 +260,39 @@ class ValidateModelOneVideo(ConfigReader, PlottingMixin, TrainModelMixin):
             cv2.putText(
                 frame,
                 "Timer",
-                (10, int(self.settings["styles"]["space_scale"])),
+                (
+                    TextOptions.BORDER_BUFFER_Y.value,
+                    int(self.settings["styles"]["space_scale"]),
+                ),
                 self.font,
                 self.settings["styles"]["font size"],
-                (0, 255, 0),
+                TextOptions.COLOR.value,
                 2,
             )
             addSpacer = 2
             cv2.putText(
                 frame,
                 (f"{self.clf_name} {target_timer}s"),
-                (10, self.settings["styles"]["space_scale"] * addSpacer),
+                (
+                    TextOptions.BORDER_BUFFER_Y.value,
+                    self.settings["styles"]["space_scale"] * addSpacer,
+                ),
                 self.font,
                 self.settings["styles"]["font size"],
-                (0, 0, 255),
+                TextOptions.COLOR.value,
                 2,
             )
             addSpacer += 1
             cv2.putText(
                 frame,
                 "Ensemble prediction",
-                (10, self.settings["styles"]["space_scale"] * addSpacer),
+                (
+                    TextOptions.BORDER_BUFFER_Y.value,
+                    self.settings["styles"]["space_scale"] * addSpacer,
+                ),
                 self.font,
                 self.settings["styles"]["font size"],
-                (0, 255, 0),
+                TextOptions.COLOR.value,
                 2,
             )
             addSpacer += 2
@@ -266,16 +300,19 @@ class ValidateModelOneVideo(ConfigReader, PlottingMixin, TrainModelMixin):
                 cv2.putText(
                     frame,
                     self.clf_name,
-                    (10, +self.settings["styles"]["space_scale"] * addSpacer),
+                    (
+                        TextOptions.BORDER_BUFFER_Y.value,
+                        +self.settings["styles"]["space_scale"] * addSpacer,
+                    ),
                     self.font,
                     self.settings["styles"]["font size"],
-                    (2, 166, 249),
+                    TextOptions.COLOR.value,
                     2,
                 )
                 addSpacer += 1
-            if self.create_gantt == "Gantt chart: final frame only (slightly faster)":
-                frame = np.concatenate((frame, self.gantt_img), axis=1)
-            elif self.create_gantt == "Gantt chart: video":
+            if self.create_gantt == 1:
+                frame = np.concatenate((frame, self.final_gantt_img), axis=1)
+            elif self.create_gantt == 2:
                 gantt_img = self.create_gantt_img(
                     self.bouts_df,
                     self.clf_name,
@@ -283,23 +320,12 @@ class ValidateModelOneVideo(ConfigReader, PlottingMixin, TrainModelMixin):
                     self.fps,
                     "Behavior gantt chart",
                 )
-                gantt_img = self.resize_gantt(
-                    gantt_img, self.video_meta_data["video_height"]
-                )
+                gantt_img = self.resize_gantt(gantt_img, self.video_meta_data["height"])
                 frame = np.concatenate((frame, gantt_img), axis=1)
-            elif self.create_gantt != "None":
-                frame = cv2.resize(
-                    frame,
-                    (
-                        int(self.video_meta_data["width"] + self.gantt_img.shape[1]),
-                        self.video_meta_data["height"],
-                    ),
-                )
+            frame = cv2.resize(frame, video_size, interpolation=cv2.INTER_LINEAR)
             writer.write(np.uint8(frame))
             print(
-                "Frame created: {} / {}".format(
-                    str(frm_cnt + 1), str(len(self.data_df))
-                )
+                f"Frame created: for video {self.feature_filename} ({frm_cnt + 1} / {len(self.data_df)})..."
             )
             frm_cnt += 1
 
@@ -312,12 +338,35 @@ class ValidateModelOneVideo(ConfigReader, PlottingMixin, TrainModelMixin):
             source=self.__class__.__name__,
         )
 
-    def run(self):
-        self.__run_clf()
-        if self.shortest_bout > 1:
-            self.__plug_bouts()
-        self.__save()
-        self.__create_video()
+
+# test = ValidateModelOneVideo(config_path=r'/Users/simon/Desktop/envs/simba/troubleshooting/mouse_open_field/project_folder/project_config.ini',
+#                              feature_file_path='/Users/simon/Desktop/envs/simba/troubleshooting/mouse_open_field/project_folder/csv/features_extracted/SI_DAY3_308_CD1_PRESENT.csv',
+#                              model_path='/Users/simon/Desktop/envs/simba/troubleshooting/mouse_open_field/models/generated_models/Running.sav',
+#                              discrimination_threshold=0.6,
+#                              shortest_bout=50,
+#                              settings={'pose': True, 'animal_names': True, 'styles': None},
+#                              create_gantt=None)
+# test.run()
+
+# test = ValidateModelOneVideo(config_path=r'/Users/simon/Desktop/envs/simba/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini',
+#                              feature_file_path='/Users/simon/Desktop/envs/simba/troubleshooting/two_black_animals_14bp/project_folder/csv/features_extracted/Together_1.csv',
+#                              model_path='/Users/simon/Desktop/envs/simba/troubleshooting/two_black_animals_14bp/models/generated_models/Attack.sav',
+#                              discrimination_threshold=0.6,
+#                              shortest_bout=50,
+#                              settings={'pose': True, 'animal_names': True, 'styles': None},
+#                              create_gantt=2)
+# test.run()
+
+
+# test = ValidateModelOneVideoMultiprocess(config_path=r'/Users/simon/Desktop/envs/simba/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini',
+#                              feature_file_path='/Users/simon/Desktop/envs/simba/troubleshooting/two_black_animals_14bp/project_folder/csv/features_extracted/Together_1.csv',
+#                              model_path='/Users/simon/Desktop/envs/simba/troubleshooting/two_black_animals_14bp/models/generated_models/Attack.sav',
+#                              discrimination_threshold=0.6,
+#                              shortest_bout=50,
+#                              cores=6,
+#                              settings={'pose': True, 'animal_names': True, 'styles': None},
+#                              create_gantt=None)
+# test.run()
 
 
 # test = ValidateModelOneVideo(config_path=r'/Users/simon/Desktop/envs/troubleshooting/dam_nest-c-only_ryan/project_folder/project_config.ini',
