@@ -5,7 +5,7 @@ from typing import Dict, Union
 
 import numpy as np
 import pandas as pd
-from scipy.stats import f_oneway
+from scipy.stats import f_oneway, kruskal
 from statsmodels.stats.libqsturng import psturng
 from statsmodels.stats.multicomp import pairwise_tukeyhsd
 
@@ -35,15 +35,17 @@ PEARSON = "pearson"
 ANOVA = "anova"
 KENDALL = "kendall"
 SHAP = "shap"
-SCALED = 'scaled'
+SCALED = "scaled"
 PLOTS = "plots"
 CREATE = "create"
 SPEARMAN = "spearman"
+KRUSKAL_WALLIS = "kruskal_wallis"
 MEAN = "MEAN"
 STDEV = "STANDARD DEVIATION"
 PERMUTATION_IMPORTANCE = "permutation_importance"
 DESCRIPTIVE_STATISTICS = "descriptive_statistics"
 ANOVA_HEADERS = ["FEATURE NAME", "F-STATISTIC", "P-VALUE"]
+KRUSKAL_HEADERS = ["FEATURE NAME", "KRUSKAL-WALLIS H STATISTIC", "P-VALUE"]
 
 
 class ClusterFrequentistCalculator(UnsupervisedMixin, ConfigReader):
@@ -73,15 +75,33 @@ class ClusterFrequentistCalculator(UnsupervisedMixin, ConfigReader):
         UnsupervisedMixin.__init__(self)
         self.settings = settings
         self.data = read_pickle(data_path=data_path)
-        self.save_path = os.path.join(self.logs_path,f"cluster_descriptive_statistics_{self.data[Clustering.CLUSTER_MODEL.value][Unsupervised.HASHED_NAME.value]}_{self.datetime}.xlsx",)
-        check_if_keys_exist_in_dict(data=self.data, key=[Clustering.CLUSTER_MODEL.value, Unsupervised.METHODS.value], name=data_path)
-        check_if_keys_exist_in_dict(data=settings, key=[SCALED, ANOVA, DESCRIPTIVE_STATISTICS, TUKEY], name="settings")
+        self.save_path = os.path.join(
+            self.logs_path,
+            f"cluster_descriptive_statistics_{self.data[Clustering.CLUSTER_MODEL.value][Unsupervised.HASHED_NAME.value]}_{self.datetime}.xlsx",
+        )
+        check_if_keys_exist_in_dict(
+            data=self.data,
+            key=[Clustering.CLUSTER_MODEL.value, Unsupervised.METHODS.value],
+            name=data_path,
+        )
+        check_if_keys_exist_in_dict(
+            data=settings,
+            key=[SCALED, ANOVA, DESCRIPTIVE_STATISTICS, TUKEY],
+            name="settings",
+        )
 
     def run(self):
-        self.x_data = self.data[Unsupervised.METHODS.value][Unsupervised.SCALED_DATA.value]
-        self.cluster_data = self.data[Clustering.CLUSTER_MODEL.value][Unsupervised.MODEL.value].labels_
+        self.x_data = self.data[Unsupervised.METHODS.value][
+            Unsupervised.SCALED_DATA.value
+        ]
+        self.cluster_data = self.data[Clustering.CLUSTER_MODEL.value][
+            Unsupervised.MODEL.value
+        ].labels_
         if not self.settings[SCALED]:
-            self.x_data = TrainModelMixin.scaler_inverse_transform(data=self.x_data, scaler=self.data[Unsupervised.METHODS.value][Unsupervised.SCALER.value])
+            self.x_data = TrainModelMixin.scaler_inverse_transform(
+                data=self.x_data,
+                scaler=self.data[Unsupervised.METHODS.value][Unsupervised.SCALER.value],
+            )
         self.x_y_df = pd.concat(
             [
                 self.x_data,
@@ -107,6 +127,9 @@ class ClusterFrequentistCalculator(UnsupervisedMixin, ConfigReader):
             self.__descriptive_stats()
         if self.settings[TUKEY]:
             self.__tukey_posthoc()
+        if self.settings[KRUSKAL_WALLIS]:
+            self.__kruskal_wallis()
+
         self.timer.stop_timer()
         stdout_success(
             msg=f"Cluster statistics complete. Data saved at {self.save_path}",
@@ -202,7 +225,39 @@ class ClusterFrequentistCalculator(UnsupervisedMixin, ConfigReader):
             elapsed_time=timer.elapsed_time_str,
         )
 
+    def __kruskal_wallis(self):
+        timer = SimbaTimer(start=True)
+        print("Calculating Kruskal-Wallis...")
+        results = pd.DataFrame(columns=KRUSKAL_HEADERS)
+        for feature_name in self.data[Unsupervised.METHODS.value][
+            Unsupervised.FEATURE_NAMES.value
+        ]:
+            feature_data = []
+            for i in self.x_y_df[CLUSTER].unique():
+                feature_data.append(
+                    list(self.x_y_df[feature_name][self.x_y_df[CLUSTER] == i].values)
+                )
+            statistic, p_val = kruskal(*feature_data)
+            results.loc[len(results)] = [feature_name, statistic, p_val]
+        results = (
+            results.reset_index(drop=True)
+            .set_index(FEATURE_NAME)
+            .sort_values("P-VALUE", ascending=True)
+        )
+        self.__save_results(df=results, name=KRUSKAL_WALLIS)
+        timer.stop_timer()
+        stdout_success(
+            msg=f"Kruskal-Wallis statistics saved in {self.save_path}",
+            elapsed_time=timer.elapsed_time_str,
+        )
 
-# settings = {'scaled': True, 'anova': True, 'tukey_posthoc': True, 'descriptive_statistics': True}
+        # data = pairwise_tukeyhsd(self.x_y_df[feature_name], self.x_y_df[CLUSTER])
+
+
+# settings = {'scaled': True,
+#             'anova': False,
+#             'tukey_posthoc': False,
+#             'descriptive_statistics': False,
+#             'kruskal_wallis': True}
 # calculator = ClusterFrequentistCalculator(config_path='/Users/simon/Desktop/envs/NG_Unsupervised/project_folder/project_config.ini', data_path='/Users/simon/Desktop/envs/NG_Unsupervised/project_folder/small_clusters/adoring_hoover.pickle', settings=settings)
-# calculator.run()
\ No newline at end of file
+# calculator.run()
