@@ -1,11 +1,11 @@
 __author__ = "Simon Nilsson"
 
 import os.path
-from typing import Union
+from typing import Tuple, Union
 
 import numpy as np
 import pandas as pd
-from numba import jit, prange, njit
+from numba import jit, njit, prange, types
 from numba.typed import List
 from scipy.sparse import csgraph
 from scipy.sparse.csgraph import minimum_spanning_tree
@@ -13,11 +13,12 @@ from scipy.sparse.csgraph import minimum_spanning_tree
 from simba.mixins.config_reader import ConfigReader
 from simba.mixins.unsupervised_mixin import UnsupervisedMixin
 from simba.unsupervised.enums import Clustering, Unsupervised
-from simba.utils.checks import (check_file_exist_and_readable, check_if_dir_exists, check_valid_extension)
-from simba.utils.printing import SimbaTimer, stdout_success, stdout_warning
-from simba.utils.read_write import get_unique_values_in_iterable, read_pickle, find_files_of_filetypes_in_directory
+from simba.utils.checks import (check_file_exist_and_readable,
+                                check_if_dir_exists, check_valid_extension)
 from simba.utils.enums import Formats
-
+from simba.utils.printing import SimbaTimer, stdout_success, stdout_warning
+from simba.utils.read_write import (find_files_of_filetypes_in_directory,
+                                    get_unique_values_in_iterable, read_pickle)
 
 CLUSTERER_NAME = "CLUSTERER_NAME"
 CLUSTER_COUNT = "CLUSTER_COUNT"
@@ -48,18 +49,23 @@ class DBCVCalculator(UnsupervisedMixin, ConfigReader):
     >>> results = dbcv_calculator.run()
     """
 
-    def __init__(self,
-                 config_path: Union[str, os.PathLike],
-                 data_path: Union[str, os.PathLike]):
-
+    def __init__(
+        self, config_path: Union[str, os.PathLike], data_path: Union[str, os.PathLike]
+    ):
         check_file_exist_and_readable(file_path=config_path)
         ConfigReader.__init__(self, config_path=config_path)
         UnsupervisedMixin.__init__(self)
         if os.path.isdir(data_path):
             check_if_dir_exists(in_dir=data_path)
-            self.data_paths = find_files_of_filetypes_in_directory(directory=data_path, extensions=[f'.{Formats.PICKLE.value}'], raise_error=True)
+            self.data_paths = find_files_of_filetypes_in_directory(
+                directory=data_path,
+                extensions=[f".{Formats.PICKLE.value}"],
+                raise_error=True,
+            )
         else:
-            check_valid_extension(path=data_path, accepted_extensions=Formats.PICKLE.value)
+            check_valid_extension(
+                path=data_path, accepted_extensions=Formats.PICKLE.value
+            )
             self.data_paths = [data_path]
         self.save_path = os.path.join(self.logs_path, f"DBCV_{self.datetime}.xlsx")
         with pd.ExcelWriter(self.save_path, mode="w") as writer:
@@ -71,22 +77,51 @@ class DBCVCalculator(UnsupervisedMixin, ConfigReader):
         for file_cnt, file_path in enumerate(self.data_paths):
             model_timer = SimbaTimer(start=True)
             v = read_pickle(data_path=file_path)
-            self.results[file_cnt], dbcv_results = {}, "nan"
-            self.results[file_cnt][CLUSTERER_NAME] = v[Clustering.CLUSTER_MODEL.value][Unsupervised.HASHED_NAME.value]
-            self.results[file_cnt][EMBEDDER_NAME] = v[Unsupervised.DR_MODEL.value][Unsupervised.HASHED_NAME.value]
-            print(f"Performing DBCV for cluster model {self.results[file_cnt][CLUSTERER_NAME]}...")
-            cluster_lbls = v[Clustering.CLUSTER_MODEL.value][Unsupervised.MODEL.value].labels_
+            self.results[file_cnt], dbcv_results, warning = {}, "nan", False
+            self.results[file_cnt][CLUSTERER_NAME] = v[Clustering.CLUSTER_MODEL.value][
+                Unsupervised.HASHED_NAME.value
+            ]
+            self.results[file_cnt][EMBEDDER_NAME] = v[Unsupervised.DR_MODEL.value][
+                Unsupervised.HASHED_NAME.value
+            ]
+            print(
+                f"Performing DBCV for cluster model {self.results[file_cnt][CLUSTERER_NAME]}..."
+            )
+            cluster_lbls = v[Clustering.CLUSTER_MODEL.value][
+                Unsupervised.MODEL.value
+            ].labels_
             x = v[Unsupervised.DR_MODEL.value][Unsupervised.MODEL.value].embedding_
-            self.results[file_cnt] = {**self.results[file_cnt], **v[Clustering.CLUSTER_MODEL.value][Unsupervised.PARAMETERS.value], **v[Unsupervised.DR_MODEL.value][Unsupervised.PARAMETERS.value]}
-            cluster_cnt = get_unique_values_in_iterable(data=cluster_lbls, name=v[Clustering.CLUSTER_MODEL.value][Unsupervised.HASHED_NAME.value], min=1)
+            self.results[file_cnt] = {
+                **self.results[file_cnt],
+                **v[Clustering.CLUSTER_MODEL.value][Unsupervised.PARAMETERS.value],
+                **v[Unsupervised.DR_MODEL.value][Unsupervised.PARAMETERS.value],
+            }
+            cluster_cnt = get_unique_values_in_iterable(
+                data=cluster_lbls,
+                name=v[Clustering.CLUSTER_MODEL.value][Unsupervised.HASHED_NAME.value],
+                min=1,
+            )
             if cluster_cnt > 1:
-                dbcv_results = self.DBCV(x.astype(np.float32), cluster_lbls.astype(np.int64))
-            self.results[file_cnt] = {**self.results[file_cnt], **{DBCV: dbcv_results}, **{CLUSTER_COUNT: cluster_cnt}}
+                dbcv_results, warning = self.DBCV(
+                    x.astype(np.float32), cluster_lbls.astype(np.int64)
+                )
+            self.results[file_cnt] = {
+                **self.results[file_cnt],
+                **{DBCV: dbcv_results},
+                **{CLUSTER_COUNT: cluster_cnt},
+                **{"WARNING (POTENTIALLY INACCURATE)": str(warning)},
+            }
             model_timer.stop_timer()
-            stdout_success(msg=f"DBCV complete for model {self.results[file_cnt][CLUSTERER_NAME]} ...", elapsed_time=model_timer.elapsed_time_str)
+            stdout_success(
+                msg=f"DBCV complete for model {self.results[file_cnt][CLUSTERER_NAME]} ...",
+                elapsed_time=model_timer.elapsed_time_str,
+            )
         self.__save_results()
         self.timer.stop_timer()
-        stdout_success(msg=f"ALL DBCV calculations complete and saved in {self.save_path}", elapsed_time=self.timer.elapsed_time_str)
+        stdout_success(
+            msg=f"ALL DBCV calculations complete and saved in {self.save_path}",
+            elapsed_time=self.timer.elapsed_time_str,
+        )
 
     def __save_results(self):
         for k, v in self.results.items():
@@ -101,14 +136,14 @@ class DBCVCalculator(UnsupervisedMixin, ConfigReader):
         :returns float: DBCV cluster validity score
         """
 
-        print('Computing mutual reach distance ... (Step  1/4)')
+        print("Computing mutual reach distance ... (Step  1/4)")
         arrays_by_cluster, ordered_labels = self._mutual_reach_dist_graph(X, labels)
-        print('Computing pairwise distances ... (Step  2/4)')
-        graph = self.calculate_dists(X, arrays_by_cluster)
-        print('Computing minimum spanning tree ... (Step  3/4)')
+        print("Computing pairwise distances ... (Step  2/4)")
+        graph, warning = self.calculate_dists(X, arrays_by_cluster)
+        print("Computing minimum spanning tree ... (Step  3/4)")
         mst = self._mutual_reach_dist_MST(graph)
-        print('Computing cluster validity index ... (Step  4/4)')
-        return self._clustering_validity_index(mst, ordered_labels)
+        print("Computing cluster validity index ... (Step  4/4)")
+        return self._clustering_validity_index(mst, ordered_labels), warning
 
     @staticmethod
     @njit("(float32[:,:], int64[:])")
@@ -140,16 +175,19 @@ class DBCVCalculator(UnsupervisedMixin, ConfigReader):
 
     @staticmethod
     @jit(nopython=True, fastmath=True)
-    def calculate_dists(X: np.ndarray, arrays_by_cluster: List):
+    def calculate_dists(
+        X: np.ndarray, arrays_by_cluster: types.List
+    ) -> Tuple[np.ndarray, bool]:
         """
         :param np.ndarray X: 2D array of shape len(observations) x len(dimensionality reduced dimensions)
         :param numba.types.ListType[list[np.ndarray]] arrays_by_cluster: Numba typed List of list with 2d arrays.
-        :returns np.ndarray graph: Graph of all pair-wise mutual reachability distances between points of size X.shape[0] x X.shape[0].
+        :returns Tuple[np.ndarray, bool): Graph of all pair-wise mutual reachability distances between points of size X.shape[0] x X.shape[0].
+                                          Boolean representing if any issues where detected. Including: If (i) any clusters consist of a single observation.
         """
-
         cluster_n = len(arrays_by_cluster)
         graph = np.empty((X.shape[0], X.shape[0]), X.dtype)
         graph_row_counter = 0
+        warning_flag = False
 
         for cluster_a in range(cluster_n):
             A = arrays_by_cluster[cluster_a][0]
@@ -166,9 +204,13 @@ class DBCVCalculator(UnsupervisedMixin, ConfigReader):
                 indices = np.where(Ci != 0)
                 Ci = Ci[indices]
                 numerator = ((1 / Ci) ** A.shape[1]).sum()
+                if A.shape[0] < 2:
+                    warning_flag = True
                 denominator_axis_0 = np.max(np.array([2, A.shape[0]]))
 
-                core_dist_i = (numerator / (denominator_axis_0 - 1)) ** (-1 / A.shape[1])
+                core_dist_i = (numerator / (denominator_axis_0 - 1)) ** (
+                    -1 / A.shape[1]
+                )
                 graph_row = np.zeros((0))
 
                 for array_cnt in range(len(arrays_by_cluster)):
@@ -184,17 +226,23 @@ class DBCVCalculator(UnsupervisedMixin, ConfigReader):
                             Cii[j] = np.sqrt(acc)
                         indices_ii = np.where(Cii != 0)
                         Cii = Cii[indices_ii]
-                        numerator = np.max(np.array([1, ((1 / Cii) ** B.shape[1]).sum()]))
+                        numerator = np.max(
+                            np.array([1, ((1 / Cii) ** B.shape[1]).sum()])
+                        )
                         denominator_axis_0 = np.max(np.array(([2, B.shape[0]])))
-                        core_dist_j = (numerator / (denominator_axis_0 - 1)) ** (-1 / B.shape[1])
+                        core_dist_j = (numerator / (denominator_axis_0 - 1)) ** (
+                            -1 / B.shape[1]
+                        )
                         dist = np.linalg.norm(A[a] - B[b])
-                        mutual_reachability = np.max(np.array([core_dist_i, core_dist_j, dist]))
+                        mutual_reachability = np.max(
+                            np.array([core_dist_i, core_dist_j, dist])
+                        )
                         graph_row = np.append(graph_row, mutual_reachability)
 
                 graph[graph_row_counter] = graph_row
                 graph_row_counter += 1
 
-        return graph
+        return graph, warning_flag
 
     def _mutual_reach_dist_MST(self, graph: np.ndarray) -> np.ndarray:
         """
@@ -266,7 +314,7 @@ class DBCVCalculator(UnsupervisedMixin, ConfigReader):
 # test.run()
 
 # test = DBCVCalculator(config_path='/Users/simon/Desktop/envs/NG_Unsupervised/project_folder/project_config.ini',
-#                       data_path='/Users/simon/Desktop/envs/NG_Unsupervised/project_folder/large_clusters')
+#                       data_path='/Users/simon/Desktop/envs/NG_Unsupervised/project_folder/small_clusters')
 # test.run()
 
 #
