@@ -46,7 +46,7 @@ class UmapEmbedder(UnsupervisedMixin):
     :param hyper_parameters: dict holding UMAP hyperparameters in list format.
 
     :Example I: Fit.
-    >>> hyper_parameters = {'n_neighbors': [10, 2], 'min_distance': [1.0], 'spread': [1.0], 'scaler': 'MIN-MAX', 'variance': 0.25}
+    >>> hyper_parameters = {'n_neighbors': [10, 2], 'min_distance': [1.0], 'spread': [1.0], 'scaler': 'MIN-MAX', 'variance': 0.25, "multicolinearity": 0.5}
     >>> data_path = 'unsupervised/project_folder/logs/unsupervised_data_20230416145821.pickle'
     >>> save_dir = 'unsupervised/dr_models'
     >>> config_path = 'unsupervised/project_folder/project_config.ini'
@@ -99,6 +99,8 @@ class UmapEmbedder(UnsupervisedMixin):
                 variance_threshold=hyper_parameters[Unsupervised.VARIANCE.value],
             )
             self.umap_df = drop_df_fields(data=self.umap_df, fields=self.low_var_cols)
+        # if hyper_parameters[Unsupervised.MULTICOLLINEARITY.value] > 0:
+        #     print(hyper_parameters[Unsupervised.MULTICOLLINEARITY.value])
         self.scaler = TrainModelMixin.define_scaler(
             scaler_name=hyper_parameters[Unsupervised.SCALER.value]
         )
@@ -290,12 +292,21 @@ class UmapEmbedder(UnsupervisedMixin):
             return results
 
 
-# hyper_parameters = {'n_neighbors': [5], 'min_distance': [0.1, 0.5, 0.0], 'spread': [1.0], 'scaler': 'MIN-MAX', 'variance': 0.25}
-# data_path = '/Users/simon/Desktop/envs/NG_Unsupervised/project_folder/logs/unsupervised_data_20240325092459.pickle'
-# save_dir = '/Users/simon/Desktop/envs/NG_Unsupervised/project_folder/small_embeddings'
-# config_path = '/Users/simon/Desktop/envs/NG_Unsupervised/project_folder/project_config.ini'
-# embedder = UmapEmbedder()
-# embedder.fit(data_path=data_path, save_dir=save_dir, hyper_parameters=hyper_parameters)
+hyper_parameters = {
+    "n_neighbors": [5],
+    "min_distance": [0.1, 0.5, 0.0],
+    "spread": [1.0],
+    "scaler": "MIN-MAX",
+    "variance": 0.25,
+    "multicollinearity": 0.7,
+}
+data_path = "/Users/simon/Desktop/envs/NG_Unsupervised/project_folder/logs/unsupervised_data_20240325092459.pickle"
+save_dir = "/Users/simon/Desktop/envs/NG_Unsupervised/project_folder/embedding_2"
+config_path = (
+    "/Users/simon/Desktop/envs/NG_Unsupervised/project_folder/project_config.ini"
+)
+embedder = UmapEmbedder()
+embedder.fit(data_path=data_path, save_dir=save_dir, hyper_parameters=hyper_parameters)
 #
 
 # data_path = '/Users/simon/Desktop/envs/simba/troubleshooting/NG_Unsupervised/project_folder/logs/unsupervised_data_20240215143716.pickle'
@@ -332,7 +343,7 @@ class UmapEmbedder(UnsupervisedMixin):
 #
 #
 #
-
+#
 # hyper_parameters = {'n_neighbors': [10, 2], 'min_distance': [1.0], 'spread': [1.0], 'scaler': 'MIN-MAX', 'variance': 0.25}
 # data_path = '/Users/simon/Desktop/envs/simba/troubleshooting/NG_Unsupervised/project_folder/logs/unsupervised_data_20240214093117.pickle'
 # save_dir = '/Users/simon/Desktop/envs/simba/troubleshooting/NG_Unsupervised/project_folder/dim_reduction_mdls'
