@@ -14,11 +14,14 @@ from simba.unsupervised.dataset_creator import DatasetCreator
 from simba.unsupervised.enums import UMLOptions, Unsupervised
 from simba.unsupervised.pop_ups.cluster_frequentist_stats_popup import \
     ClusterFrequentistStatisticsPopUp
+from simba.unsupervised.pop_ups.cluster_validation_pop_up import \
+    ClusterValidatorPopUp
 from simba.unsupervised.pop_ups.cluster_videos_popup import \
     ClusterVisualizerPopUp
 from simba.unsupervised.pop_ups.cluster_xai_popup import ClusterXAIPopUp
+from simba.unsupervised.pop_ups.clusterer_comparison_popup import \
+    ClustererComparisonPopUp
 from simba.unsupervised.pop_ups.data_extractor_popup import DataExtractorPopUp
-from simba.unsupervised.pop_ups.dbcv_popup import DBCVPopUp
 from simba.unsupervised.pop_ups.embedding_correlations_popup import \
     EmbedderCorrelationsPopUp
 from simba.unsupervised.pop_ups.fit_cluster_popup import FitClusterModelsPopUp
@@ -234,9 +237,9 @@ class UnsupervisedGUI(ConfigReader, PopUpMixin):
         )
         self.dbcv_btn = Button(
             self.metrics_frm,
-            text="DENSITY-BASED CLUSTER VALIDATION",
+            text="CLUSTER VALIDATIONS",
             fg="blue",
-            command=lambda: DBCVPopUp(config_path=self.config_path),
+            command=lambda: ClusterValidatorPopUp(config_path=self.config_path),
         )
         self.extract_single_metrics_btn = Button(
             self.metrics_frm,
@@ -264,6 +267,14 @@ class UnsupervisedGUI(ConfigReader, PopUpMixin):
             fg="orange",
             command=lambda: EmbedderCorrelationsPopUp(config_path=self.config_path),
         )
+
+        self.clusterer_comparisons_btn = Button(
+            self.metrics_frm,
+            text="CLUSTERER COMPARISONS",
+            fg="green",
+            command=lambda: ClustererComparisonPopUp(config_path=self.config_path),
+        )
+
         self.print_embedding_info_btn = Button(
             self.metrics_frm,
             text="PRINT MODEL INFO",
@@ -277,7 +288,8 @@ class UnsupervisedGUI(ConfigReader, PopUpMixin):
         self.cluster_descriptives_btn.grid(row=2, column=0, sticky="NW")
         self.cluster_xai_btn.grid(row=3, column=0, sticky="NW")
         self.embedding_corr_btn.grid(row=4, column=0, sticky="NW")
-        self.print_embedding_info_btn.grid(row=5, column=0, sticky="NW")
+        self.clusterer_comparisons_btn.grid(row=5, column=0, sticky="NW")
+        self.print_embedding_info_btn.grid(row=6, column=0, sticky="NW")
 
         self.main_frm.mainloop()
 
