@@ -8,6 +8,7 @@ import subprocess
 from pathlib import Path
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 
+import cv2
 import numpy as np
 import pandas as pd
 import trafaret as t
@@ -15,13 +16,14 @@ import trafaret as t
 from simba.utils.enums import Keys, Options, UMAPParam
 from simba.utils.errors import (ArrayError, ColumnNotFoundError,
                                 CorruptedFileError, CountError,
-                                DirectoryNotEmptyError, FloatError,
-                                IntegerError, InvalidFilepathError,
-                                InvalidInputError, NoDataError,
-                                NoFilesFoundError, NoROIDataError,
+                                DirectoryNotEmptyError, FFMPEGNotFoundError,
+                                FloatError, FrameRangeError, IntegerError,
+                                InvalidFilepathError, InvalidInputError,
+                                NoDataError, NoFilesFoundError, NoROIDataError,
                                 NotDirectoryError, ParametersFileError,
                                 StringError)
-from simba.utils.warnings import NoDataFoundWarning
+from simba.utils.warnings import (CorruptedFileWarning, FrameRangeWarning,
+                                  NoDataFoundWarning)
 
 
 def check_file_exist_and_readable(file_path: Union[str, os.PathLike]) -> None:
@@ -304,18 +306,35 @@ def check_all_file_names_are_represented_in_video_log(
         )
 
 
-def check_if_dir_exists(in_dir: Union[str, os.PathLike]) -> None:
+def check_if_dir_exists(
+    in_dir: Union[str, os.PathLike],
+    source: Optional[str] = None,
+    create_if_not_exist: Optional[bool] = False,
+) -> None:
     """
     Check if a directory path exists.
 
-    :param str in_dir: Putative directory path.
+    :param Union[str, os.PathLike] in_dir: Putative directory path.
+    :param Optional[str] source: String source for interpretable error messaging.
+    :param Optional[bool] create_if_not_exist: If directory does not exist, then create it. Default False,
     :raise NotDirectoryError: The directory does not exist.
     """
+
     if not os.path.isdir(in_dir):
-        raise NotDirectoryError(
-            msg=f"{in_dir} is not a valid directory",
-            source=check_if_dir_exists.__name__,
-        )
+        if create_if_not_exist:
+            try:
+                os.makedirs(in_dir)
+            except:
+                pass
+        if source is None:
+            raise NotDirectoryError(
+                msg=f"{in_dir} is not a valid directory",
+                source=check_if_dir_exists.__name__,
+            )
+        else:
+            raise NotDirectoryError(
+                msg=f"{in_dir} is not a valid directory", source=source
+            )
 
 
 def check_that_column_exist(
@@ -507,17 +526,24 @@ def check_nvidea_gpu_available() -> bool:
         return False
 
 
-def check_ffmpeg_available() -> bool:
+def check_ffmpeg_available(raise_error: Optional[bool] = False) -> Union[bool, None]:
     """
     Helper to check of FFMpeg is available via subprocess ``ffmpeg``.
 
-    returns bool: True if ``ffmpeg`` returns not None. Else False.
+    :param Optional[bool] raise_error: If True, raises ``FFMPEGNotFoundError`` if FFmpeg can't be found. Else return False. Default False.
+    :returns bool: True if ``ffmpeg`` returns not None and raise_error is False. Else False.
     """
+
     try:
         subprocess.call("ffmpeg", stderr=subprocess.DEVNULL, stdout=subprocess.DEVNULL)
         return True
     except Exception:
-        return False
+        if raise_error:
+            raise FFMPEGNotFoundError(
+                msg="FFMpeg could not be found on the instance (as evaluated via subprocess ffmpeg). Please make sure FFMpeg is installed."
+            )
+        else:
+            return False
 
 
 def check_if_valid_rgb_str(
@@ -688,8 +714,9 @@ def check_if_module_has_import(parsed_file: ast.Module, import_name: str) -> boo
     return False
 
 
-def check_valid_extension(path: Union[str, os.PathLike],
-                          accepted_extensions: Union[List[str], str]):
+def check_valid_extension(
+    path: Union[str, os.PathLike], accepted_extensions: Union[List[str], str]
+):
     """
     Checks if the file extension of the provided path is in the list of accepted extensions.
 
@@ -697,20 +724,29 @@ def check_valid_extension(path: Union[str, os.PathLike],
     :param List[str] accepted_extensions: A list of accepted file extensions. E.g., ['pickle', 'csv'].
     """
     if isinstance(accepted_extensions, (list, tuple)):
-        check_valid_lst(data=accepted_extensions, source=f'{check_valid_extension.__name__} accepted_extensions', valid_dtypes=(str,), min_len=1)
+        check_valid_lst(
+            data=accepted_extensions,
+            source=f"{check_valid_extension.__name__} accepted_extensions",
+            valid_dtypes=(str,),
+            min_len=1,
+        )
     elif isinstance(accepted_extensions, str):
-        check_str(name=f'{check_valid_extension.__name__} accepted_extensions', value=accepted_extensions)
+        check_str(
+            name=f"{check_valid_extension.__name__} accepted_extensions",
+            value=accepted_extensions,
+        )
         accepted_extensions = [accepted_extensions]
     accepted_extensions = [x.lower() for x in accepted_extensions]
     check_file_exist_and_readable(file_path=path)
     extension = get_fn_ext(filepath=path)[2][1:]
     if extension.lower() not in accepted_extensions:
-        raise InvalidFilepathError(msg=f'File extension for file {path} has an invalid extension. Found {extension}, accepted: {accepted_extensions}', source=check_valid_extension.__name__)
+        raise InvalidFilepathError(
+            msg=f"File extension for file {path} has an invalid extension. Found {extension}, accepted: {accepted_extensions}",
+            source=check_valid_extension.__name__,
+        )
 
 
-def check_if_valid_img(
-    data: np.ndarray, source: Optional[str] = "", raise_error: Optional[bool] = True
-) -> Union[bool, None]:
+def check_if_valid_img(data: np.ndarray, source: Optional[str] = "", raise_error: Optional[bool] = True) -> Union[bool, None]:
     """
     Check if a variable is a valid image.
 
@@ -719,9 +755,7 @@ def check_if_valid_img(
     :parameter Optional[bool] raise_error: If True, raise InvalidInputError if invalid image representation. Else, return bool.
     """
 
-    check_instance(
-        source=check_if_valid_img.__name__, instance=data, accepted_types=np.ndarray
-    )
+    check_instance(source=check_if_valid_img.__name__, instance=data, accepted_types=np.ndarray)
     if (data.ndim != 2) and (data.ndim != 3):
         if raise_error:
             raise InvalidInputError(
@@ -763,19 +797,18 @@ def check_that_dir_has_list_of_filenames(
             )
 
 
-def check_valid_array(
-    data: np.ndarray,
-    source: Optional[str] = "",
-    accepted_ndims: Optional[List[Tuple[int]]] = None,
-    accepted_sizes: Optional[List[int]] = None,
-    accepted_axis_0_shape: Optional[List[int]] = None,
-    accepted_axis_1_shape: Optional[List[int]] = None,
-    accepted_dtypes: Optional[List[str]] = None,
-    accepted_shapes: Optional[List[Tuple[int]]] = None,
-    min_axis_0: Optional[int] = None,
-    max_axis_1: Optional[int] = None,
-    min_axis_1: Optional[int] = None,
-) -> None:
+def check_valid_array(data: np.ndarray,
+                      source: Optional[str] = "",
+                      accepted_ndims: Optional[List[Tuple[int]]] = None,
+                      accepted_sizes: Optional[List[int]] = None,
+                      accepted_axis_0_shape: Optional[List[int]] = None,
+                      accepted_axis_1_shape: Optional[List[int]] = None,
+                      accepted_dtypes: Optional[List[str]] = None,
+                      accepted_values: Optional[List[Any]] = None,
+                      accepted_shapes: Optional[List[Tuple[int]]] = None,
+                      min_axis_0: Optional[int] = None,
+                      max_axis_1: Optional[int] = None,
+                      min_axis_1: Optional[int] = None) -> None:
     """
     Check if the given  array satisfies specified criteria regarding its dimensions, shape, and data type.
 
@@ -852,11 +885,24 @@ def check_valid_array(
     if max_axis_1 is not None:
         check_int(name=f"{source} max_axis_1", value=max_axis_1)
         if data.shape[1] > max_axis_1:
-            raise ArrayError(msg=f"Array not of acceptable shape. Found  {data.shape[1]} columns, maximum columns accepted: {max_axis_1}, {source}", source=check_valid_array.__name__)
+            raise ArrayError(
+                msg=f"Array not of acceptable shape. Found  {data.shape[1]} columns, maximum columns accepted: {max_axis_1}, {source}",
+                source=check_valid_array.__name__,
+            )
     if min_axis_1 is not None:
-        check_int(name=f"{source} min_axis_1", value=max_axis_1)
+        check_int(name=f"{source} min_axis_1", value=min_axis_1)
         if data.shape[1] < min_axis_1:
-            raise ArrayError(msg=f"Array not of acceptable shape. Found  {data.shape[1]} columns, minimum columns accepted: {min_axis_1}, {source}", source=check_valid_array.__name__)
+            raise ArrayError(
+                msg=f"Array not of acceptable shape. Found  {data.shape[1]} columns, minimum columns accepted: {min_axis_1}, {source}",
+                source=check_valid_array.__name__,
+            )
+
+    if accepted_values is not None:
+        check_valid_lst(data=accepted_values, source=f"{source} accepted_values")
+        additional_vals = list(set(np.unique(data)) - set(accepted_values))
+        if len(additional_vals) > 0:
+            raise ArrayError(msg=f"Array contains unacceptable values. Found  {additional_vals}, accepted: {accepted_values}, {source}", source=check_valid_array.__name__,)
+
 
 def check_valid_lst(
     data: list,
@@ -982,7 +1028,10 @@ def check_if_keys_exist_in_dict(
     for k in key:
         if k not in list(data.keys()):
             if raise_error:
-                raise InvalidInputError(msg=f"{k} does not exist in object {name}",source=check_if_keys_exist_in_dict.__class__.__name__)
+                raise InvalidInputError(
+                    msg=f"{k} does not exist in object {name}",
+                    source=check_if_keys_exist_in_dict.__class__.__name__,
+                )
         else:
             pass
     return True
@@ -1119,5 +1168,228 @@ def check_video_has_rois(roi_dict: dict, video_names: List[str], roi_names: List
         if len(missing_rois) > 0:
             raise NoROIDataError(
                 msg=f"{len(missing_rois)} ROI(s) are missing from {video_name}: {missing_rois}",
-                source=spontaneous_alternations.__name__,
+                source=check_video_has_rois.__name__,
             )
+
+
+def check_if_df_field_is_boolean(df: pd.DataFrame,
+                                 field: str,
+                                 raise_error: Optional[bool] = True,
+                                 bool_values: Optional[Tuple[Any]] = (0, 1),
+                                 df_name: Optional[str] = ''):
+    """Helper to check if a dataframe field is a boolean value"""
+    check_instance(source=f'{check_if_df_field_is_boolean.__name__} df', instance=df, accepted_types=(pd.DataFrame,))
+    check_str(name=f"{check_if_df_field_is_boolean.__name__} field", value=field)
+    check_that_column_exist(df=df, column_name=field, file_name=check_if_df_field_is_boolean.__name__)
+    additional = list((set(list(df[field])) - set(bool_values)))
+    if len(additional) > 0:
+        if raise_error:
+            raise CountError(
+                msg=f"Field {field} not a boolean in {df_name}. Found values {additional}. Accepted: {bool_values}",
+                source=check_if_df_field_is_boolean.__name__,
+            )
+        else:
+            return False
+    return True
+
+
+def check_valid_dataframe(
+    df: pd.DataFrame,
+    source: Optional[str] = "",
+    valid_dtypes: Optional[Tuple[Any]] = None,
+    required_fields: Optional[List[str]] = None,
+    min_axis_0: Optional[int] = None,
+    min_axis_1: Optional[int] = None,
+    max_axis_0: Optional[int] = None,
+    max_axis_1: Optional[int] = None,
+):
+    """Helper to check if a dataframe is valid"""
+    check_instance(source=source, instance=df, accepted_types=(pd.DataFrame,))
+    if valid_dtypes is not None:
+        dtypes = list(set(df.dtypes))
+        additional = [x for x in dtypes if x not in valid_dtypes]
+        if len(additional) > 0:
+            raise InvalidInputError(
+                msg=f"The dataframe {source} has invalid data format(s) {additional}. Valid: {valid_dtypes}",
+                source=source,
+            )
+    if min_axis_1 is not None:
+        check_int(name=f"{source} min_axis_1", value=min_axis_1, min_value=1)
+        if len(df.columns) < min_axis_1:
+            raise InvalidInputError(
+                msg=f"The dataframe {source} has less than ({df.columns}) the required minimum number of columns ({min_axis_1}).",
+                source=source,
+            )
+    if min_axis_0 is not None:
+        check_int(name=f"{source} min_axis_0", value=min_axis_0, min_value=1)
+        if len(df) < min_axis_0:
+            raise InvalidInputError(
+                msg=f"The dataframe {source} has less than ({len(df)}) the required minimum number of rows ({min_axis_0}).",
+                source=source,
+            )
+    if max_axis_0 is not None:
+        check_int(name=f"{source} max_axis_0", value=min_axis_0, min_value=1)
+        if len(df) > max_axis_0:
+            raise InvalidInputError(
+                msg=f"The dataframe {source} has more than ({len(df)}) the required maximum number of rows ({max_axis_0}).",
+                source=source,
+            )
+    if max_axis_1 is not None:
+        check_int(name=f"{source} max_axis_1", value=min_axis_1, min_value=1)
+        if len(df.columns) > max_axis_1:
+            raise InvalidInputError(
+                msg=f"The dataframe {source} has more than ({df.columns}) the required maximum number of columns ({max_axis_1}).",
+                source=source,
+            )
+    if required_fields is not None:
+        check_valid_lst(
+            data=required_fields,
+            source=check_valid_dataframe.__name__,
+            valid_dtypes=(str,),
+        )
+        missing = list(set(required_fields) - set(df.columns))
+        if len(missing) > 0:
+            raise InvalidInputError(
+                msg=f"The dataframe {source} are missing required columns {missing}.",
+                source=source,
+            )
+
+
+def check_valid_tuple(
+    x: tuple,
+    source: Optional[str] = "",
+    accepted_lengths: Optional[Tuple[int]] = None,
+    valid_dtypes: Optional[Tuple[Any]] = None,
+):
+    if not isinstance(x, (tuple)):
+        raise InvalidInputError(
+            msg=f"{check_valid_tuple.__name__} {source} is not a valid tuple",
+            source=source,
+        )
+    if accepted_lengths is not None:
+        if len(x) not in accepted_lengths:
+            raise InvalidInputError(
+                msg=f"Tuple is not of valid lengths. Found {len(x)}. Accepted: {accepted_lengths}",
+                source=source,
+            )
+    if valid_dtypes is not None:
+        dtypes = list(set([type(v) for v in x]))
+        additional = [x for x in dtypes if x not in valid_dtypes]
+        if len(additional) > 0:
+            raise InvalidInputError(
+                msg=f"The tuple {source} has invalid data format(s) {additional}. Valid: {valid_dtypes}",
+                source=source,
+            )
+
+
+def check_video_and_data_frm_count_align(
+    video: Union[str, os.PathLike, cv2.VideoCapture],
+    data: Union[str, os.PathLike, pd.DataFrame],
+    name: Optional[str] = "",
+    raise_error: Optional[bool] = True,
+) -> None:
+    """
+    Check if the frame count of a video matches the row count of a data file.
+
+    :param Union[str, os.PathLike, cv2.VideoCapture] video: Path to the video file or cv2.VideoCapture object.
+    :param Union[str, os.PathLike, pd.DataFrame] data: Path to the data file or DataFrame containing the data.
+    :param Optional[str] name: Name of the video (optional for interpretable error msgs).
+    :param Optional[bool] raise_error: Whether to raise an error if the counts don't align (default is True). If False, prints warning.
+    :return None:
+
+    :example:
+    >>> data_1 = '/Users/simon/Desktop/envs/simba/troubleshooting/mouse_open_field/project_folder/csv/outlier_corrected_movement_location/SI_DAY3_308_CD1_PRESENT.csv'
+    >>> video_1 = '/Users/simon/Desktop/envs/simba/troubleshooting/mouse_open_field/project_folder/frames/output/ROI_analysis/SI_DAY3_308_CD1_PRESENT.mp4'
+    >>> check_video_and_data_frm_count_align(video=video_1, data=data_1, raise_error=True)
+    """
+
+    def _count_generator(reader):
+        b = reader(1024 * 1024)
+        while b:
+            yield b
+            b = reader(1024 * 1024)
+
+    check_instance(
+        source=f"{check_video_and_data_frm_count_align.__name__} video",
+        instance=video,
+        accepted_types=(str, cv2.VideoCapture),
+    )
+    check_instance(
+        source=f"{check_video_and_data_frm_count_align.__name__} data",
+        instance=data,
+        accepted_types=(str, pd.DataFrame),
+    )
+    check_str(
+        name=f"{check_video_and_data_frm_count_align.__name__} name",
+        value=name,
+        allow_blank=True,
+    )
+    if isinstance(video, str):
+        check_file_exist_and_readable(file_path=video)
+        video = cv2.VideoCapture(video)
+    video_count = int(video.get(cv2.CAP_PROP_FRAME_COUNT))
+    if isinstance(data, str):
+        check_file_exist_and_readable(file_path=data)
+        with open(data, "rb") as fp:
+            c_generator = _count_generator(fp.raw.read)
+            data_count = (sum(buffer.count(b"\n") for buffer in c_generator)) - 1
+    else:
+        data_count = len(data)
+    if data_count != video_count:
+        if not raise_error:
+            FrameRangeWarning(msg=f"The video {name} has {video_count} frames, but the associated data file for this video has {data_count} rows", source=check_video_and_data_frm_count_align.__name__)
+        else:
+            raise FrameRangeError(
+                msg=f"The video {name} has {video_count} frames, but the associated data file for this video has {data_count} rows",
+                source=check_video_and_data_frm_count_align.__name__,
+            )
+
+
+def check_if_video_corrupted(video: Union[str, os.PathLike, cv2.VideoCapture],
+                             frame_interval: Optional[int] = None,
+                             frame_n: Optional[int] = 20,
+                             raise_error: Optional[bool] = True) -> None:
+
+    """
+    Check if a video file is corrupted by inspecting a set of its frames.
+
+    .. note::
+       For decent run-time regardless of video length, pass a smaller ``frame_n`` (<100).
+
+    :param Union[str, os.PathLike] video_path: Path to the video file or cv2.VideoCapture OpenCV object.
+    :param Optional[int] frame_interval: Interval between frames to be checked. If None, ``frame_n`` will be used.
+    :param Optional[int] frame_n: Number of frames to be checked, will be sampled at large allowed interval. If None, ``frame_interval`` will be used.
+    :param Optional[bool] raise_error: Whether to raise an error if corruption is found. If False, prints warning.
+    :return None:
+
+    :example:
+    >>> check_if_video_corrupted(video_path='/Users/simon/Downloads/NOR ENCODING FExMP8.mp4')
+    """
+    check_instance(source=f'{check_if_video_corrupted.__name__} video', instance=video, accepted_types=(str, cv2.VideoCapture))
+    if isinstance(video, str):
+        check_file_exist_and_readable(file_path=video)
+        cap = cv2.VideoCapture(video)
+    else:
+        cap = video
+    frame_count = int(video.get(cv2.CAP_PROP_FRAME_COUNT))
+    if (frame_interval is not None and frame_n is not None) or (frame_interval is None and frame_n is None):
+        raise InvalidInputError(msg='Pass frame_interval OR frame_n', source=check_if_video_corrupted.__name__)
+    if frame_interval is not None:
+        frms_to_check = list(range(0, frame_count, frame_interval))
+    else:
+        frms_to_check = np.array_split(np.arange(0, frame_count), frame_n)
+        frms_to_check = [x[-1] for x in frms_to_check]
+    errors = []
+    for frm_id in frms_to_check:
+        cap.set(1, frm_id)
+        ret, _ = cap.read()
+        if not ret: errors.append(frm_id)
+    if len(errors) > 0:
+        if raise_error:
+            raise CorruptedFileError(msg=f'Found {len(errors)} corrupted frame(s) at indexes {errors} in video {video}', source=check_if_video_corrupted.__name__)
+        else:
+            CorruptedFileWarning(msg=f'Found {len(errors)} corrupted frame(s) at indexes {errors} in video {video}', source=check_if_video_corrupted.__name__)
+    else:
+        pass
+
+
