@@ -2,7 +2,6 @@ __author__ = "Simon Nilsson"
 
 import ast
 import configparser
-import glob
 import io
 import os
 import subprocess
@@ -26,12 +25,14 @@ except:
 
 from simba.utils.checks import (check_file_exist_and_readable, check_float,
                                 check_if_dir_exists,
+                                check_if_keys_exist_in_dict,
                                 check_if_module_has_import,
                                 check_if_string_value_is_valid_video_timestamp,
                                 check_instance, check_int, check_str,
+                                check_that_column_exist,
                                 check_that_hhmmss_start_is_before_end,
-                                check_valid_array, check_that_column_exist)
-from simba.utils.enums import ConfigKey, Dtypes, Options
+                                check_valid_array, check_valid_dataframe)
+from simba.utils.enums import ConfigKey, Dtypes, Keys, Options
 from simba.utils.errors import (BodypartColumnNotFoundError, CountError,
                                 InvalidFileTypeError, InvalidInputError,
                                 NoFilesFoundError)
@@ -647,6 +648,41 @@ def convert_roi_definitions(
             )
 
 
+def slice_roi_dict_for_video(
+    data: Dict[str, pd.DataFrame], video_name: str
+) -> Tuple[Dict[str, pd.DataFrame], List[str]]:
+    """
+    Given a dictionary of dataframes representing different ROIs (created by ``simba.mixins.config_reader.ConfigReader.read_roi_data``),
+    retain only the ROIs belonging to the specified video.
+    """
+    check_if_keys_exist_in_dict(
+        data=data,
+        key=[
+            Keys.ROI_RECTANGLES.value,
+            Keys.ROI_CIRCLES.value,
+            Keys.ROI_POLYGONS.value,
+        ],
+        name=slice_roi_dict_for_video.__name__,
+    )
+    new_data, shape_names = {}, []
+    for k, v in data.items():
+        check_instance(
+            source=f"{slice_roi_dict_for_video.__name__} {k}",
+            instance=v,
+            accepted_types=(pd.DataFrame,),
+        )
+        check_that_column_exist(
+            df=v, column_name="Video", file_name=slice_roi_dict_for_video.__name__
+        )
+        check_that_column_exist(
+            df=v, column_name="Name", file_name=slice_roi_dict_for_video.__name__
+        )
+        v = v[v["Video"] == video_name]
+        new_data[k] = v.reset_index(drop=True)
+        shape_names.extend((list(v["Name"].unique())))
+    return new_data, shape_names
+
+
 def freedman_diaconis(data: np.array) -> (float, int):
     """
     Use Freedman-Diaconis rule to compute optimal count of histogram bins and their width.
@@ -937,9 +973,17 @@ def sample_df_n_by_unique(df: pd.DataFrame, field: str, n: int) -> pd.DataFrame:
     :return pd.DataFrame: A dataframe containing randomly sampled rows.
     """
 
-    check_instance(source=sample_df_n_by_unique.__name__, instance=df, accepted_types=(pd.DataFrame,))
-    check_str(name=f'{sample_df_n_by_unique.__name__} field', value=field, options=tuple(df.columns))
-    check_int(name=f'{sample_df_n_by_unique.__name__} n', value=n, min_value=1)
+    check_instance(
+        source=sample_df_n_by_unique.__name__,
+        instance=df,
+        accepted_types=(pd.DataFrame,),
+    )
+    check_str(
+        name=f"{sample_df_n_by_unique.__name__} field",
+        value=field,
+        options=tuple(df.columns),
+    )
+    check_int(name=f"{sample_df_n_by_unique.__name__} n", value=n, min_value=1)
     check_that_column_exist(df=df, column_name=field, file_name="")
     unique_vals = df[field].unique()
     results = []
@@ -951,8 +995,9 @@ def sample_df_n_by_unique(df: pd.DataFrame, field: str, n: int) -> pd.DataFrame:
             results.append(sample.sample(n=n, replace=False))
     return pd.concat(results, axis=0)
 
+
 def get_mode(x: np.ndarray) -> Union[float, int]:
-    """ Get the mode (most frequent value) within an aarray  """
+    """Get the mode (most frequent value) within an array"""
     check_valid_array(
         source=f"{get_mode.__name__} x",
         data=x,
