@@ -208,6 +208,7 @@ class Options(Enum):
         "1980×1080",
     ]
     DPI_OPTIONS = [100, 200, 400, 800, 1600, 3200]
+    RESOLUTION_OPTIONS_2 = ["AUTO", 240, 320, 480, 640, 720, 800, 960, 1120, 1080, 1980]
     SPEED_OPTIONS = [round(x, 1) for x in list(np.arange(0.1, 2.1, 0.1))]
     PERFORM_FLAGS = ["yes", True, "True"]
     RUN_OPTIONS_FLAGS = ["yes", True, "True", "False", "no", False, "true", "false"]
@@ -267,7 +268,7 @@ class Options(Enum):
     SMOOTHING_OPTIONS_W_NONE = ["None", "Gaussian", "Savitzky Golay"]
     VIDEO_FORMAT_OPTIONS = ["mp4", "avi"]
     ALL_VIDEO_FORMAT_OPTIONS = (".avi", ".mp4", ".mov", ".flv", ".m4v")
-    ALL_IMAGE_FORMAT_OPTIONS = (".bmp", ".png", ".jpeg", ".jpg")
+    ALL_IMAGE_FORMAT_OPTIONS = (".bmp", ".png", ".jpeg", ".jpg", ".webp")
     ALL_IMAGE_FORMAT_STR_OPTIONS = ".bmp .png .jpeg .jpg"
     ALL_VIDEO_FORMAT_STR_OPTIONS = ".avi .mp4 .mov .flv .m4v"
     WORKFLOW_FILE_TYPE_OPTIONS = ["csv", "parquet"]
@@ -369,6 +370,7 @@ class TextOptions(Enum):
 class Defaults(Enum):
     MAX_TASK_PER_CHILD = 10
     LARGE_MAX_TASK_PER_CHILD = 1000
+    MAXIMUM_MAX_TASK_PER_CHILD = 8000
     CHUNK_SIZE = 1
     SPLASH_TIME = 2500
     try:
