@@ -8,6 +8,7 @@ import pickle
 import platform
 import re
 import shutil
+import subprocess
 import threading
 import webbrowser
 from configparser import ConfigParser
@@ -428,7 +429,6 @@ def get_video_meta_data(
     :example:
     >>> get_video_meta_data('test_data/video_tests/Video_1.avi')
     {'video_name': 'Video_1', 'fps': 30, 'width': 400, 'height': 600, 'frame_count': 300, 'resolution_str': '400 x 600', 'video_length_s': 10}
-
     """
 
     video_data = {}
@@ -466,6 +466,7 @@ def concatenate_videos_in_folder(
     substring: Optional[str] = None,
     remove_splits: Optional[bool] = True,
     gpu: Optional[bool] = False,
+    verbose: Optional[bool] = True,
 ) -> None:
     """
     Concatenate (temporally) all video files in a folder into a single video.
@@ -522,13 +523,9 @@ def concatenate_videos_in_folder(
     if os.path.exists(save_path):
         os.remove(save_path)
     if check_nvidea_gpu_available() and gpu:
-        returned = os.system(
-            f'ffmpeg -hwaccel auto -c:v h264_cuvid -f concat -safe 0 -i "{temp_txt_path}" -c copy -hide_banner -loglevel info "{save_path}"'
-        )
+        returned = os.system(f'ffmpeg -hwaccel auto -c:v h264_cuvid -f concat -safe 0 -i "{temp_txt_path}" -c copy -hide_banner -loglevel info "{save_path}"')
     else:
-        returned = os.system(
-            f'ffmpeg -f concat -safe 0 -i "{temp_txt_path}" "{save_path}" -c copy -hide_banner -loglevel info'
-        )
+        returned = os.system(f'ffmpeg -f concat -safe 0 -i "{temp_txt_path}" "{save_path}" -c copy -hide_banner -loglevel info')
     while True:
         if returned != 0:
             pass
@@ -633,11 +630,12 @@ def find_all_videos_in_directory(
             raise NoFilesFoundError(
                 f"No videos found in directory {directory} in formats {video_formats}."
             )
-        video_lst.append("No videos found")
-        NoFileFoundWarning(
-            msg=f"No videos found in directory ({directory})",
-            source=find_all_videos_in_directory.__name__,
-        )
+        else:
+            video_lst.append("No videos found")
+            NoFileFoundWarning(
+                msg=f"No videos found in directory ({directory})",
+                source=find_all_videos_in_directory.__name__,
+            )
 
     if video_lst and as_dict:
         video_dict = {}
@@ -1573,10 +1571,9 @@ def find_max_vertices_coordinates(
 
     return int(max_x), int(max_y)
 
-
 def clean_sleap_file_name(filename: str) -> str:
     """
-    Clean a SLEAP input filename by removing '.analysis' suffix and project name prefix to match orginal video name.
+    Clean a SLEAP input filename by removing '.analysis' suffix, the video number, and project name prefix, to match orginal video name.
 
      .. note::
        Modified from `vtsai881 <https://github.com/vtsai881>`_.
@@ -1591,11 +1588,13 @@ def clean_sleap_file_name(filename: str) -> str:
     >>> 'videoname.h5'
     """
 
-    SLEAP_CSV_SUBSTR = ".analysis"
-    if (SLEAP_CSV_SUBSTR in filename) and ("_" in filename):
-        return filename.replace(filename.split("_")[0] + "_", "").replace(
-            SLEAP_CSV_SUBSTR, ""
-        )
+    if (".analysis" in filename.lower()) and ("_" in filename) and (filename.count('.') >= 3):
+        filename_parts = filename.split('.')
+        video_num_name = filename_parts[2]
+        if '_' in video_num_name:
+            return video_num_name.split('_', 1)[1]
+        else:
+            return filename
     else:
         return filename
 
@@ -1750,38 +1749,56 @@ def write_pickle(data: Dict[str, Any], save_path: Union[str, os.PathLike]) -> No
         )
 
 
-def read_pickle(data_path: Union[str, os.PathLike]) -> dict:
+def read_pickle(
+    data_path: Union[str, os.PathLike], verbose: Optional[bool] = False
+) -> dict:
     """
     Read a single or directory of pickled objects. If directory, returns dict with numerical sequential integer keys for
     each object.
 
     :param str data_path: Pickled file path, or directory of pickled files.
+    :param Optional[bool] verbose: If True, prints progress. Default False.
     :returns dict
 
     :example:
     >>> data = read_pickle(data_path='/test/unsupervised/cluster_models')
     """
     if os.path.isdir(data_path):
-        print(f"Reading in data directory {data_path}...")
+        if verbose:
+            print(f"Reading in data directory {data_path}...")
         data = {}
         files_found = glob.glob(data_path + f"/*.{Formats.PICKLE.value}")
         if len(files_found) == 0:
             raise NoFilesFoundError(
-                msg=f"SIMBA ERROR: Zero pickle files found in {data_path}.", source=read_pickle.__name__)
+                msg=f"SIMBA ERROR: Zero pickle files found in {data_path}.",
+                source=read_pickle.__name__,
+            )
         for file_cnt, file_path in enumerate(files_found):
+            if verbose:
+                _, file_name, _ = get_fn_ext(filepath=file_path)
+                print(f"Reading in data file {file_name}...")
             with open(file_path, "rb") as f:
                 try:
                     data[file_cnt] = pickle.load(f)
                 except Exception as e:
                     print(e.args)
-                    raise InvalidFileTypeError(msg=f'Could not decompress file {file_path} - invalid pickle', source=read_pickle.__name__)
+                    raise InvalidFileTypeError(
+                        msg=f"Could not decompress file {file_path} - invalid pickle",
+                        source=read_pickle.__name__,
+                    )
     elif os.path.isfile(data_path):
+        if verbose:
+            _, file_name, _ = get_fn_ext(filepath=data_path)
+            print(f"Reading in data file {file_name}...")
         with open(data_path, "rb") as f:
             try:
                 data = pickle.load(f)
             except Exception as e:
                 print(e.args)
-                raise InvalidFileTypeError(msg=f'Could not decompress file {data_path} - invalid pickle', source=read_pickle.__name__)
+                raise InvalidFileTypeError(
+                    msg=f"Could not decompress file {data_path} - invalid pickle",
+                    source=read_pickle.__name__,
+                )
     else:
         raise InvalidFilepathError(
             msg=f"The path {data_path} is neither a valid file or directory path",
@@ -1876,8 +1893,100 @@ def get_unique_values_in_iterable(
     return cnt
 
 
-# def kbd_listener() -> KeyboardListener:
-#     kbd_listener = KeyboardListener()
-#     monitor_thread = threading.Thread(target=kbd_listener.start)
-#     monitor_thread.start()
-#     return kbd_listener
+def copy_files_to_directory(
+    file_paths: List[Union[str, os.PathLike]],
+    dir: Union[str, os.PathLike],
+    verbose: Optional[bool] = True,
+    integer_save_names: Optional[bool] = False,
+) -> List[Union[str, os.PathLike]]:
+    """
+    Copy a list of files to a specified directory.
+
+    :param List[Union[str, os.PathLike]] file_paths: List of paths to the files to be copied.
+    :param Union[str, os.PathLike] dir: Path to the directory where files will be copied.
+    :param Optional[bool] verbose: If True, prints progress information. Default True.
+    :param Optional[bool] integer_save_names: If True, saves files with integer names. E.g., file one in ``file_paths`` will be saved as dir/0.
+    :return List[Union[str, os.PathLike]]: List of paths to the copied files
+    """
+
+    check_valid_lst(data=file_paths, source=copy_files_to_directory.__name__, min_len=1)
+    _ = [check_file_exist_and_readable(x) for x in file_paths]
+    check_if_dir_exists(in_dir=dir, source=copy_files_to_directory)
+    destinations = []
+    for cnt, file_path in enumerate(file_paths):
+        if verbose:
+            print(
+                f"Copying file {os.path.basename(file_path)} ({cnt+1}/{len(file_paths)})..."
+            )
+        if not integer_save_names:
+            destination = os.path.join(dir, os.path.basename(file_path))
+        else:
+            _, file_name, ext = get_fn_ext(filepath=file_path)
+            destination = os.path.join(dir, f"{cnt}{ext}")
+        try:
+            if os.path.isfile(destination):
+                os.remove(destination)
+        except Exception as e:
+            print(e.args)
+            raise PermissionError(
+                msg=f"Not allowed to overwrite file {destination}. Try running SimBA in terminal opened in admin mode or delete existing file before copying.",
+                source=copy_files_to_directory.__name__,
+            )
+        destinations.append(destination)
+        shutil.copy(file_path, destination)
+    return destinations
+
+
+def seconds_to_timestamp(seconds: int) -> str:
+    """
+    Convert an integer number representing seconds to a HH:MM:SS format.
+    """
+    check_int(
+        name=f"{seconds_to_timestamp.__name__} seconds", value=seconds, min_value=0
+    )
+    hours = int(seconds / 3600)
+    minutes = int((seconds % 3600) / 60)
+    seconds = int(seconds % 60)
+    return "{:02d}:{:02d}:{:02d}".format(hours, minutes, seconds)
+
+
+def read_data_paths(path: Union[str, os.PathLike, None],
+                    default: List[Union[str, os.PathLike]],
+                    default_name: Optional[str] = "",
+                    file_type: Optional[str] = "csv") -> List[str]:
+    """
+    Helper to flexibly read in a set of file-paths.
+
+    :param Union[str, os.PathLike] path: None or path to a file or a folder or list of paths to files.
+    :param List[Union[str, os.PathLike]] default: If ``path`` is None. Use this passed list of file paths.
+    :param Optional[str] default_name: A readable name representing the ``default`` for interpretable error msgs. Defaults to empty string.
+    :param Optional[str] file_type: If path is a directory, read in all files in directory with this file extension. Default: ``csv``.
+    :return List[str]: List of file paths.
+    """
+
+    if path is None:
+        if len(default) == 0:
+            raise NoFilesFoundError(msg=f"No files in format found in {default_name}", source=read_data_paths.__name__)
+        else:
+            for i in default:
+                check_file_exist_and_readable(file_path=i)
+            data_paths = default
+    elif isinstance(path, str):
+        if os.path.isfile(path):
+            check_file_exist_and_readable(file_path=path)
+            data_paths = [path]
+        elif os.path.isdir(path):
+            data_paths = find_files_of_filetypes_in_directory(directory=path, extensions=[f".{file_type}"], raise_error=True)
+            if len(data_paths) == 0:
+                raise NoFilesFoundError(msg=f"No files in format {file_type} found in {default_name}", source=read_data_paths.__name__)
+        else:
+            raise NoFilesFoundError(msg=f"{path} is not a valid path string (it's nether a file or a folder)", source=read_data_paths.__name__)
+    elif isinstance(path, (list, tuple)):
+        check_valid_lst(data=path, source=f"{read_data_paths.__name__} path", valid_dtypes=(str,), min_len=1)
+        data_paths = []
+        for i in path:
+            check_file_exist_and_readable(file_path=i)
+            data_paths.append(i)
+    else:
+        raise NoFilesFoundError(msg=f"{type(path)} is not a valid type for path", source=read_data_paths.__name__)
+    return data_paths
